//================================================================================
//  
//      Complex.cs      fZ
//        (c)2012/08/28  by OjisanKoubou
//  
//================================================================================
//  
//  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
//  ľړIɂ́ACAȂǎRɎgpł܂B
//  \ۂ͒쌠\Ȃ悤ɂĂB
//  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷Ȃǂ
//  ƓIp͂ł܂B
//

using System;

public struct Complex {
    private readonly double re;
    private readonly double im;


    public Complex(double re, double im) {
        this.re = re;
        this.im = im;
    }


    public double Re {
        get { return re; }
    }
    
    
    public double Im {
        get { return im; }
    }
    
    
    public static Complex operator  + (Complex c1, Complex c2) {
        return new Complex(c1.re + c2.re, c1.im + c2.im);
    }
    
    
    public static Complex operator - (Complex c1, Complex c2) {
        return new Complex(c1.re - c2.re, c1.im - c2.im);
    }
    
    
    public static Complex operator - (Complex c1) {
        return new Complex(-c1.re, -c1.im);
    }
    
    
    public static Complex operator * (Complex c1, Complex c2) {
        return new Complex( c1.re * c2.re - c1.im * c2.im, c1.re * c2.im + c1.im * c2.re );
    }
    
    
    public static Complex operator * (Complex c, double d) {
        return new Complex( c.re * d, c.im * d );
    }
    
    
    public static Complex operator * (double d, Complex c) {
        return new Complex( c.re * d, c.im * d );
    }
    
    
    public static Complex operator  / (Complex c1, Complex c2) {
        double d = c2.re * c2.re + c2.im * c2.im;
        if (d == 0) throw new DivideByZeroException("Divide by zero Complex number");
        return new Complex( (c1.re * c2.re + c1.im * c2.im) / d, (c1.im * c2.re - c1.re * c2.im) / d );
    }
    
    
    public static Complex operator  / (Complex c1, double d) {
        if (d == 0) throw new DivideByZeroException("Divide by zero Complex number");
        return new Complex( c1.re/d, c1.im/d );
    }
    
    
    public double Abs {
        get{ return Math.Sqrt(re * re + im * im); }
    }
    
    
    public double Decibel {
        get{ return 10*Math.Log10(re * re + im * im); }
    }
    
    
    public double Arg {
        get{ return Math.Atan2(im, re)*180/Math.PI; }
    }
    
    
    public override string ToString() {
         return String.Format("({0}, {1}i)", re, im);
    }
    
}

