﻿//================================================================================
//  
//        DemodForm.cs      復調器画面
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり
//  販売したりするなど商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace APB_3 {
    public partial class DemodForm : Form {
        private static int wn=0;
        private MainForm mainf;
        private Thread TAgc;
        private volatile bool measuring = false;
        private volatile bool setting = false;
        private volatile bool agcsuspend = false;
        private volatile bool agcsuspended = false;
        private int mod = 0;
        private bool initializing = true;
        
        private int[] bandtable = new int[17]{ 5,4,4,4,4,3,3,3,3,2,2,2,2,1,1,1,1};
        private int[] tune1table = new int[17]{ 0,179,132, 69, 22,160, 92, 46, 11,180,109, 62, 12,152,100, 63,  7 };
        private int[] tune2table = new int[17]{ 0,195,150,130,100,180,160,140,120,200,130, 80, 40,210,120, 60,  8 };    // C1=100p  C2=100p
        private int[] tune3table = new int[17]{ 0,192,138, 71, 24,160, 95, 46, 10,190,111, 62, 13,155,100, 64, 11 };
        
        private delegate void DelegateType();     // 別スレッドのコントロールを操作するのに使う
        
        
        public DemodForm() {
            mainf = Apb3.mainf;
            wn++;
            
            InitializeComponent();
            this.Text = "復調器 " + wn.ToString();
            
            comboBox_freq.Items.AddRange(def.demodfreqstrtbl);
            comboBox_freqstep.Items.AddRange(def.demodfreqstepstrtbl);
            comboBox_gain.Items.AddRange(def.demodgainstrtbl); 
            comboBox_bw.Items.AddRange(def.demodbwstrtbl);
            comboBox_mod.Items.AddRange(def.demodmodstrtbl);
            comboBox_prefreq.Items.AddRange(def.prefreqstrtbl);
            
            comboBox_freq.SelectedIndex = 15;       // 954kHz TBS
            comboBox_freqstep.SelectedIndex = 7;    // 9kHz
            comboBox_gain.SelectedIndex = 1;        // 0dB
            comboBox_bw.SelectedIndex = 4;          // 10kHz
            comboBox_mod.SelectedIndex = 2;         // AM
            comboBox_preamp.SelectedIndex = 19;     // 
            comboBox_prefreq.SelectedIndex = 0;     // 
            comboBox_band.SelectedIndex = 2;        // 
            comboBox_tune1.SelectedIndex = 0;       // 
            comboBox_tune2.SelectedIndex = 0;       // 
            comboBox_tune3.SelectedIndex = 0;       // 
            
            Setting();
            Thread.Sleep(1);
            initializing = false;
        }
        
        
        // 設定
        private void Setting() {
            if( measuring ) {
mainf.LogPrint("\nDemodForm.Setting: measuring");
                agcsuspend = true;
                int timeout=100;
                while( !agcsuspended && (timeout--)!=0 ) Thread.Sleep(10);  // TAgc が停止するのを待つ
                if( timeout==0 ) {      // タイムアウトしたときは設定しない
                    agcsuspend = false;
                    return;
                }
            }
                if( Apb3.Open() ) {
                    double freq = util.ToDouble(comboBox_freq.Text);
                    int bw      = comboBox_bw.SelectedIndex;
                    int gain    = comboBox_gain.SelectedIndex;
                    mod         = comboBox_mod.SelectedIndex;
                    
                    int preampgain = util.ToInt32(comboBox_preamp.Text);
                    int band = def.prebandtbl[util.ToInt32(comboBox_band.Text)];
                    int tune1 = util.ToInt32(comboBox_tune1.Text);
                    int tune2 = util.ToInt32(comboBox_tune2.Text);
                    int tune3 = util.ToInt32(comboBox_tune3.Text);
                    
                    
                    Apb3.uda1345_OutSelect( 3, 3 );              // Lch = Rch = DEMOD
                    Apb3.dds0( freq );                           // 変換周波数設定
                    
                    //Apb3.pcm1754_Write((0<<4) + 0x0202);        // ゲインが大きすぎてブツ音がでないよう gain=0 にしておく
                    //Apb3.dds( 0, freq );                        // 変換周波数設定
                    Apb3.CICWrite( bw+8 );                      // BW によってデシメーション設定
                    
                    //Apb3.Preamp(band, preampgain, tune1, tune2, tune3, 0, 0);  // プリアンプ設定
                    
                    if( gain == 0 ) {  // AGC
                        Apb3.Close();
                        agcsuspend = false;
                        if( !measuring ) {
                            TAgc = new Thread(new ThreadStart(Agc));
                            TAgc.Start();                 // AGCスレッドを開始する
                        }
                    } else {
                        Apb3.demod_setting( gain-1, mod );
                        Apb3.Close();
                    }
                }
                else {
                }
                setting = false;
        }
        
        
        // イベント　設定を変更した
        private void SettingChanged(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.SettingChanged");
            setting = true;
            if( !initializing ) Setting();
        }
        
        
        private void comboBox_freq_Changed(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.comboBox_freq_Changed");
            setting = true;
            if( checkBox_autofreqadj.Checked ) {  // 受信周波数に連動してバンド、チューニング電圧を変える
                double freq = util.ToDouble(comboBox_freq.Text);
                for( int i=1; i<def.prefreqstrtbl.Count()-1; i++ ) {
                    double freq_h = util.ToDouble(def.prefreqstrtbl[i]);
                    double freq_l = util.ToDouble(def.prefreqstrtbl[i+1]);
                    if( (freq <= freq_h)&&(freq >= freq_l) ) {
                        int band    = bandtable[i];
                        int tune1_h = tune1table[i];
                        int tune1_l = tune1table[i+1];
                        int tune2_h = tune2table[i];
                        int tune2_l = tune2table[i+1];
                        int tune3_h = tune3table[i];
                        int tune3_l = tune3table[i+1];
                        int tune1 = (int)(tune1_l + (tune1_h - tune1_l)*(freq - freq_l)/(freq_h - freq_l));
                        int tune2 = (int)(tune2_l + (tune2_h - tune2_l)*(freq - freq_l)/(freq_h - freq_l));
                        int tune3 = (int)(tune3_l + (tune3_h - tune3_l)*(freq - freq_l)/(freq_h - freq_l));
                        
                        comboBox_band.Text  = band.ToString();
                        comboBox_tune1.Text = tune1.ToString();
                        comboBox_tune2.Text = tune2.ToString();
                        comboBox_tune3.Text = tune3.ToString();
                    }
                }
            }
            if( !initializing ) Setting();
        }
        
        
        // イベント　周波数マイナス
        private void button_freqminus_Click(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.button_freqminus_Click");
           if( setting ) return;
            double freq  = util.ToDouble(comboBox_freq.Text);
            double fstep = util.ToDouble(comboBox_freqstep.Text);
            freq -= fstep;
            comboBox_freq.Text = util.Double2String( freq );
        }
        
        
        // イベント　周波数プラス
        private void button_freqplus_Click(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.button_freqminus_Click");
            if( setting ) return;
            double freq  = util.ToDouble(comboBox_freq.Text);
            double fstep = util.ToDouble(comboBox_freqstep.Text);
            freq += fstep;
            comboBox_freq.Text = util.Double2String( freq );
        }
        
        
        // イベント　プリアンプ周波数変更
        private void comboBox_prefreq_SelectedIndexChanged(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.comboBox_prefreq_SelectedIndexChanged");
           if( setting ) return;
            int index = comboBox_prefreq.SelectedIndex;
            if( !checkBox_autofreqadj.Checked ) {
                comboBox_band.Text = bandtable[index].ToString();
                comboBox_tune1.Text = tune1table[index].ToString();
                comboBox_tune2.Text = tune2table[index].ToString();
                comboBox_tune3.Text = tune3table[index].ToString();
            }
            if( !initializing ) Setting();
        }
        
        
        // イベント　プリアンプチューニング1変更
        private void comboBox_tune1_Changed(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.comboBox_tune1_Changed");
           if( setting ) return;
           int index = comboBox_prefreq.SelectedIndex;
           if( !checkBox_autofreqadj.Checked ) {
               tune1table[index] = util.ToInt32(comboBox_tune1.Text);
           }
           if( !initializing ) Setting();
        }
        
        
        // イベント　プリアンプチューニング2変更
        private void comboBox_tune2_Changed(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.comboBox_tune2_Changed");
           if( setting ) return;
           int index = comboBox_prefreq.SelectedIndex;
           if( !checkBox_autofreqadj.Checked ) {
               tune2table[index] = util.ToInt32(comboBox_tune2.Text);
           }
           if( !initializing ) Setting();
        }
        
        
        // イベント　プリアンプチューニング3変更
        private void comboBox_tune3_Changed(object sender, EventArgs e) {
mainf.LogPrint("\nDemodForm.comboBox_tune3_Changed");
           if( setting ) return;
           int index = comboBox_prefreq.SelectedIndex;
           if( !checkBox_autofreqadj.Checked ) {
               tune3table[index] = util.ToInt32(comboBox_tune3.Text);
           }
           if( !initializing ) Setting();
        }
        
        
        // AGC スレッド
        private void Agc() {
            if( Apb3.Open() ) {
                try {
                    measuring = true;
                    agcsuspended = false;
                    int diff = 0;
                    int level=0;
                    const int maxgain=15;
                    int gain = maxgain;  // 最大値
                    while( measuring ) {
                            if( (level = Apb3.demod_readlevel())<0 ) break;
                            if( gain >= ((maxgain-1)-level) ) {
                                gain = (maxgain-1)-level;
                                diff = 0;
                            } else {
                                if( ((maxgain-1)-(gain+level)) > 1 ) diff += (maxgain-1)-(gain+level);
                                else diff--;
                            }
                            if( diff > 10 ) {
                                gain++;
                                diff = 0;
                            }
                            if( gain > maxgain ) gain = maxgain;
                            if( gain < 0       ) gain = 0;
                            
                            BeginInvoke((DelegateType)delegate {
                                label_level.Text = level.ToString();
                                label_gain.Text = (gain*6).ToString()+"dB";
                            });
                            Apb3.demod_setting( gain, mod );

                        if( agcsuspend ) {
                            Apb3.Close();
                            agcsuspended = true;    // TAgc が停止したことを通知
                            while( agcsuspend ) Thread.Sleep(100);
                            agcsuspended = false;
                            Apb3.Open();
                        } else {
                            Thread.Sleep(200);
                        }
                    }
                }
                catch (Exception ex) {
                    mainf.LogPrint("\nDemodForm.Agc: exception=" + ex.Message);
                }
                finally {
                    Apb3.Close();
                    measuring = false;
                    agcsuspended = false;
                }
            }
        }
        
        
        
        private void DemodForm_FormClosing(object sender, FormClosingEventArgs e) {
            if( measuring ) {
                measuring = false;
                TAgc.Abort();
            }
        }
        
        
        
        private void DemodForm_Deactivate(object sender, EventArgs e) {
            if (measuring) {
                agcsuspend = true;
            }
        }
        
        
        
        private void DemodForm_Activated(object sender, EventArgs e) {
            Setting();
        }
        
        
        private void DemodForm_Click(object sender, EventArgs e) {
            this.Focus();               // どこをクリックされてもフォーカスがくるように
        }
    }
}
