﻿//================================================================================
//  
//        FMView.cs           FM復調画面
//  
//        (c)2011/07/20  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど
//  商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Collections;
using System.IO;


namespace APB_3
{
    
    //WRITE
    //
    //  0x0400  DDS0 FREQ0[ 7.. 0]    変換周波数
    //  0x0401  DDS0 FREQ0[15.. 8]
    //  0x0402  DDS0 FREQ0[23..16]
    //  0x0403  DDS0 FREQ0[31..24]
    //
    //
    //  0x0700  7 6 5 4 3 2 1 0
    //                        +-- START      1 を書くとメモリーへの書き込み開始
    //
    //  0x0701  7 6 5 4 3 2 1 0
    //                  | | | +-- RUNMODE    0: 即開始 1: ECに応じたメモリー長読み込んだ後TRGを待つ
    //                  | +-+---- EC         トリガー点　　0: 先頭  1:1/8  2:中央  3:7/8
    //                  +-------- SEL        入力切替  0:MEMDATAIN0  1:MEMDATAIN1
    //
    //
    //  0x0800  7 6 5 4 3 2 1 0
    //                        +-- START/STOP  0 を書くと測定停止、1 を書くと測定開始　
    //
    //
    //  0x0801  7 6 5 4 3 2 1 0
    //            | | | | +-+-+-- MESBITSEL   0:1/1  1:1/2    2:1/4   ...  7:1/128　
    //            | | +-+-------- PRESCALER   0:1/1  1:1/256  2:1/65536
    //            | +------------ FCINSEL     周波数カウンター入力切替  0:直接 1:周波数変換後
    //            +-------------- SEQ         0:単発測定   1: 連続測定
    //
    //  0x0802  7 6 5 4 3 2 1 0
    //          | | | | +-+-+-+-- SIGSEL     入力切替　
    //          | +-+-+---------- FMCICSEL   FM 検波器 CIC 段数切替  0:32 1:64 ... 7:4096
    //          +---------------- TRGEDGE    トリガー出力　0:立ち上がり　1:立下り
    //
    //  0x0803  7 6 5 4 3 2 1 0
    //                  +-+-+-+-- FMSEQN     何回ごとに記録するか 0:1回　1:2回　2:4回  ... 15:32768回ごと
    //                                        PHASEOV & PHASE
    //
    //  0x0900  7 6 5 4 3 2 1 0
    //                      | +-- SEL_L      0:DDS_SG_L  1:FM_DET
    //                      +---- SEL_R      0:DDS_SG_R  1:FM_DET
    //
    
    
    //READ
    //  0x0800  7 6 5 4 3 2 1 0
    //                    | +-+-- STATE      0:測定終了 1:測定開始 2:開始カウンター値取得 or 連続測定中
    //                    +------ FSIGN      周波数の符号 0:正 1:負
    //
    //  0x0801  SEQCNT   (  7 downto  0 )
    //  0x0802  MESCNT   (  7 downto  0 )
    //  0x0803  CNT_STOP (  7 downto  0 )
    //  0x0804  CNT_STOP ( 15 downto  8 )
    //  0x0805  CNT_STOP ( 23 downto 16 )
    //  0x0806  CNT_STOP ( 31 downto 24 )
    //  0x0807  CNT_START(  7 downto  0 )
    //  0x0808  CNT_START( 15 downto  8 )
    //  0x0809  CNT_START( 23 downto 16 )
    //  0x080A  CNT_START( 31 downto 24 )
    //
    //
    //  0x0700  7 6 5 4 3 2 1 0
    //                    | | +-- START      測定開始した
    //                    | +---- TRGWAIT    トリガーポジションまでデータ読み込みしトリガー受け付け可
    //                    +------ TRGD       トリガーがかかった
    //
    //  0x0701  ADRCOUNT(7 downto 0)
    //  0x0702  ADRCOUNT(11 downto 8)
    
    
    public partial class FMView : TimeGraphForm
    {
        private static int wn = 0;
        private Thread Tmeasure;
        private volatile bool measuring = false;
        private volatile bool stopmeasuring = false;
        private volatile bool errormeasuring = false;
        private int input = 0;
        private double freq = 0;
        private int bw = 0;
        private int aq = 0;
        private int triggermode = 0;
        private int triggeredge = 0;          // 0:立ち上がり　1:立下り
        private int triggerpos = 0;
        private int newtriggerpoint = -1;
        private bool isDispTime = true;       // 時間表示 or 周波数表示
        private bool initializing = true;

        private delegate void DelegateType();     // 別スレッドのコントロールを操作するのに使う


        //  コンストラクタ
        public FMView() /*(MainForm mf)*/
        {
            mainf = Apb3.mainf;
            InitializeComponent();
            base.gx = this.button_minus.Location.X; // グラフパネルの位置  dpi から計算するとずれるのでマイナスボタン位置に合わせた
            base.gy = this.button_minus.Location.Y + this.button_minus.Size.Height + 5;   
            
            wn++;
            this.Text = "FMアナライザー " + wn.ToString();
            //this.BackColor = clr.OSVPictBG;
            comboBox_input.Items.AddRange(def.FMinputstrtbl);
            comboBox_freq.Items.AddRange(def.FMfreqstrtbl);
            comboBox_freqdelta.Items.AddRange(def.FMfreqdeltastrtbl);
            comboBox_bw.Items.AddRange(def.FMbwstrtbl);
            comboBox_aq.Items.AddRange(def.FMaqstrtbl);
            comboBox_triggermode.Items.AddRange(def.FMsamplingmodestrtbl);
            comboBox_triggerpos.Items.AddRange(def.triggerposstrtbl);

            comboBox_input.SelectedIndex = 8;       // AD
            comboBox_freq.SelectedIndex = 0;        // 10MHz
            comboBox_freqdelta.SelectedIndex = 3;   // 100kHz
            comboBox_bw.SelectedIndex = 6;          // ±24k
            comboBox_aq.SelectedIndex = 0;          // 1/1
            comboBox_triggermode.SelectedIndex = 0; // フリーラン
            comboBox_triggerpos.SelectedIndex = 2;  // 中央

//            wavelist.Add(new TimeGraphWave(this, 0x3FFFFFFF, 19, 360)); // 符号付アナログ　30ビット
            wavelist.Add(new TimeGraphWave(this, 0x3FFFFFFF, 19, ClientSize.Height - (this.button_minus.Size.Height + 5) - base.gy - 18)); // 符号付アナログ　30ビット
            ((TimeGraphWave)wavelist[0]).nameedited = true;
            ((TimeGraphWave)wavelist[0]).name = "Δ 周波数";
            ((TimeGraphWave)wavelist[0]).dso_perdivtbl  = def.FMperdivtbl;
            ((TimeGraphWave)wavelist[0]).dso_perdiv     = 2e3;         // 2kHz
            ((TimeGraphWave)wavelist[0]).dso_perbit     = 47.684;      // 100M/32/65536
            ((TimeGraphWave)wavelist[0]).dso_top        = 12e3;        // 画面の一番上
            ((TimeGraphWave)wavelist[0]).dso_ndiv       = 12;          // 分割数

            wavex = 120;

            curlist.Add(new TimeCursor(this, "R", 0));
            curlist.Add(new TimeCursor(this, "M", 1));
            curcursor = ((TimeCursor)curlist[0]);
            CursorHighlight(curcursor);

            setting();
            wavelength = Apb3.buffsize/4 - 1; // 位相の差分をとる（微分）のでサンプリング数が１減る
            initwave();
            WaveRefresh();
            Multi(1);
            ReSize();
            initializing = false;
            
mainf.LogPrint("\nWidth="+ClientSize.Width + "  Height="+ClientSize.Height);
        }


        // ボタン　測定開始／終了
        private void button_startstop_Click(object sender, EventArgs e)
        {
            mainf.LogPrint("\nFMView.button_startstop_Click:");
            if (measuring)
            {
                stopmeasuring = true;
            }
            else
            {
//                if( !setting() ) return;
                
                measuring = true;
                stopmeasuring = false;
                errormeasuring = false;
                
                ((TimeGraphWave)wavelist[0]).dso_perbit = def.FMperbittbl[bw + aq];  // 感度
                clkperiod = def.FMsamplingtimetbl[bw + aq];       // サンプリングタイム
                
                Tmeasure = new Thread(new ThreadStart(tMeasure));
                Tmeasure.IsBackground = true;
                Tmeasure.Start();                 // 測定スレッドを開始する
            }
        }


        // 設定が変更された
        private void SettingChanged(object sender, EventArgs e)
        {
//            setting();
            if( !initializing ) setting();
        }


        // 周波数マイナス
        private void button_freqminus_Click(object sender, EventArgs e)
        {
            double freqdelata = util.ToDouble(comboBox_freqdelta.Text);
            double freq = util.ToDouble(comboBox_freq.Text) - freqdelata;
            comboBox_freq.Text = " " + util.Double2String(freq);
        }


        // 周波数プラス
        private void button_freqplus_Click(object sender, EventArgs e)
        {
            double freqdelata = util.ToDouble(comboBox_freqdelta.Text);
            double freq = util.ToDouble(comboBox_freq.Text) + freqdelata;
            comboBox_freq.Text = " " + util.Double2String(freq);
        }


        // ボタン　縮小
        private void button_minus_Click(object sender, EventArgs e)
        {
            MultiCursorFix(imulti - 1);
            UpdatePaint();
        }


        // ボタン　拡大
        private void button_plus_Click(object sender, EventArgs e)
        {
            MultiCursorFix(imulti + 1);
            UpdatePaint();
        }


        // ボタン　全体⇔縮小
        private void button_full_Click(object sender, EventArgs e)
        {
            base.MagFull();
        }


        // ボタン　Refカーソル
        private void button_ref_Click(object sender, EventArgs e)
        {
//            movetoCursor((TimeCursor)curlist[0]);
            curcursor = (TimeCursor)curlist[0];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }


        // ボタン　Mesカーソル
        private void button_mes_Click(object sender, EventArgs e)
        {
//            movetoCursor((TimeCursor)curlist[1]);
            curcursor = (TimeCursor)curlist[1];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }


        // ボタン　Mes-Refカーソル
        private void button_mesref_Click(object sender, EventArgs e)
        {
            int time0 = ((TimeCursor)curlist[0]).time;
            int time1 = ((TimeCursor)curlist[1]).time;
            ZoomBetween(time0, time1);
            UpdatePaint();
        }


        // コンテクストメニューOpeningイベントハンドラー
        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            contextMenuStrip1.Items.Clear();
            int waveno = ClickedWave(mousex, mousey);
            if (waveno >= 0)
            {
                cmenuWaveno = waveno;
                this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_perdiv,
                        this.ToolStripMenuItem_averaging,
                        this.ToolStripMenuItem_wavecolor,
                        this.ToolStripMenuItem_fullspan,
                        this.ToolStripMenuItem_refcursor,
                        this.ToolStripMenuItem_mescursor,
                });
                // 垂直軸感度のチェックマークをつける
                foreach (ToolStripMenuItem item in ToolStripMenuItem_perdiv.DropDownItems)
                {
                    item.Checked = false;
                }
                TimeGraphWave wave = (TimeGraphWave)wavelist[cmenuWaveno];
                ((ToolStripMenuItem)(ToolStripMenuItem_perdiv.DropDownItems[wave.dso_perdivindex])).Checked = true;
            }
            else
            {
                this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_fullspan,
                        this.ToolStripMenuItem_refcursor,
                        this.ToolStripMenuItem_mescursor,
                });
            }
        }


        // コンテキストメニュー　垂直軸感度変更
        private void ToolStripMenuItem_perdiv_Clicked(object sender, EventArgs e)
        {
            TimeGraphWave wave = (TimeGraphWave)wavelist[cmenuWaveno];
            wave.dso_perdivindex = ToolStripMenuItem_perdiv.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            UpdatePaint();
        }


        // コンテキストメニュー　平均化
        private void ToolStripMenuItem_averaging_Clicked(object sender, EventArgs e)
        {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_averaging.DropDownItems)
            {
                item.Checked = (item.Equals(sender)) ? true : false;
            }

            int index = ToolStripMenuItem_averaging.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ToolStripMenuItem_averaging.Checked = (index != 0); // 平均化しているときはチェックマークをつける

            TimeGraphWave wave = (TimeGraphWave)wavelist[cmenuWaveno];
            wave.dso_averaging = def.FMaveragingtbl[index];
            UpdatePaint();
        }


        // コンテキストメニュー　波形色
        private void ToolStripMenuItem_wavecolor_Click(object sender, EventArgs e)
        {
            WaveColor(cmenuWaveno);
        }


        // コンテキストメニュー　拡大⇔縮小
        private void ToolStripMenuItem_fullspan_Click(object sender, EventArgs e)
        {
            base.MagFull();
        }


        // コンテキストメニュー　ref カーソル
        private void ToolStripMenuItem_refcursor_Click(object sender, EventArgs e)
        {
//            base.sttime = ((TimeCursor)curlist[0]).time - (int)((gw - wavex - 18) / multi / 2);
//            base.UpdatePaint();
            curcursor = (TimeCursor)curlist[0];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }


        // コンテキストメニュー　mes カーソル
        private void ToolStripMenuItem_mescursor_Click(object sender, EventArgs e)
        {
//            base.sttime = ((TimeCursor)curlist[1]).time - (int)((gw - wavex - 18) / multi / 2);
//            base.UpdatePaint();
            curcursor = (TimeCursor)curlist[1];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }


        // コンテキストメニュー　時間表示モード⇔周波数表示モード
        private void ToolStripMenuItem_freqmode_Click(object sender, EventArgs e)
        {
            if (isDispTime)
            {
                ToolStripMenuItem_freqmode.Text = "時間表示";
                isDispTime = false;
            }
            else
            {
                ToolStripMenuItem_freqmode.Text = "周波数表示";
                isDispTime = true;
            }
            UpdateBtnCursorText();
        }
        
        
        
        // 設定
        private bool setting()
        {
            input = comboBox_input.SelectedIndex;
            freq = util.ToDouble(comboBox_freq.Text);
            bw = comboBox_bw.SelectedIndex;
            aq = comboBox_aq.SelectedIndex;
            triggermode = comboBox_triggermode.SelectedIndex;
            triggerpos = comboBox_triggerpos.SelectedIndex;
            if (triggermode==2) {
              triggermode=1;
              triggeredge = 1;
            }else if( triggermode==1) {
              triggeredge = 0;
            }
            if (triggermode == 1)
            {
                switch (triggerpos)
                {
                    case 0: newtriggerpoint =    8; break;
                    case 1: newtriggerpoint = 1024; break;
                    case 2: newtriggerpoint = 4096; break;
                    case 3: newtriggerpoint = 7168; break;
                }
            }
            else
            {
                newtriggerpoint = -10000;
            }

            if (Apb3.Open())
            {
                Apb3.dds0( freq );                                                    // 変換周波数
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.freqdet + 2, (triggeredge<<7)+input);   // トリガー、入力切替
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.freqdet + 3, aq);                       // 何回ごとに記録するか
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.cic, (bw+5));                           // CIC 段数 32-4096
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.uda1345, 0x55);                         // DAC 入力を FM 検波出力にする
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol + 1, (1 << 3) + (triggerpos << 1) + triggermode); // 入力をfreqdet
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol, 1);                         // 測定開始

                Apb3.Close();
                return true;
            }
            
            return false;
        }
        
        
        
        // 測定スレッド
        private void tMeasure()
        {
            if (Apb3.Open())
            {
                Apb3.dds0( freq );                                                    // 変換周波数
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.freqdet + 2, (triggeredge<<7)+input);   // トリガー、入力切替
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.freqdet + 3, aq);                       // 何回ごとに記録するか
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.cic, (bw+5));                           // CIC 段数 32-4096
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.uda1345, 0x55);                         // DAC 入力を FM 検波出力にする
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol + 1, (1 << 3) + (triggerpos << 1) + triggermode); // 入力をfreqdet
                Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol, 1);                         // 測定開始

                if( StartAq() )
                {
                    if( ReadData() >= 0 ) {
                        Invoke((DelegateType)delegate
                        {
//                            ZoomBetween(0, wavelength);
//                            sttime = 0;
                            triggerpoint = newtriggerpoint;
                            UpdatePaint();
                            //MagFull();
                        });
                    }else{
                        errormeasuring = true;
                    }
                }
                btnStartText(def.measure, clr.MeasBtnNor);
                
                if( errormeasuring ) {
                    mainf.LogPrint("\nFMView.tMeasure: 通信エラー");
                    error.ErrorMessageBox(error.usb_disconnect); // 測定中に通信エラー
                    Apb3.CloseConnection();     // 一度クローズする
                }
                
                Apb3.Close();
            }
            measuring=false;
        }
        
        
        
        // 
        private bool StartAq()
        {
            while (true)
            {
                int status = Apb3.Fpga.FpgaRead(Apb3.Fpga.memcontrol);
                if (status < 0) {
                    errormeasuring = true;
                    return false;  // read status ERROR ?
                }
                if ((status & 0x1) == 0) return true; // 測定終了した
                if ((status & 0x4) != 0)
                {
                    btnStartText(def.trgd, clr.MeasBtnExec);
                }
                else if ((status & 0x2) != 0)
                {
                    btnStartText(def.wait, clr.MeasBtnExec);
                }
                else if ((status & 0x1) != 0)
                {
                    btnStartText(def.pre, clr.MeasBtnExec);
                }
                
                if (stopmeasuring) return false;         // 測定中に中断した？
                
                Thread.Sleep(100);
            }
        }
        
        
        // FPGA からデータを読んで wavedata に格納する
        private int ReadData()
        {
            int ramadrs=0;
            ramadrs = Apb3.Fpga.FpgaRead(Apb3.Fpga.memcontrol+1, 2); // 最終書き込みアドレス+1

            Byte[] RBuf = new Byte[Apb3.buffsize];
            if (Apb3.Fpga.FpgaRead(Apb3.Fpga.memory, Apb3.buffsize, RBuf) != Apb3.buffsize) 
            {
                Apb3.mainf.LogPrint("\nFMView.ReadData: USB Read Error");
                return error.usb_readerr;
            }

            int prevphase = 0;
            int prevphaseov = 0;
            int diffphase = 0;
            int adrs;
            for (adrs = 0; adrs < Apb3.buffsize/4; adrs++)
            {
                ramadrs = ramadrs & 0x1FFF;
                int phase   = ((int)RBuf[ramadrs * 4    ]) + (((int)RBuf[ramadrs * 4 + 1]) << 8);
                int phaseov = ((int)RBuf[ramadrs * 4 + 2]) + (((int)RBuf[ramadrs * 4 + 3]) << 8);
                if( phase >  32767 ) phase -= 65536;
                if( phaseov >  32767 ) phaseov -= 65536;
                
                diffphase = prevphase - phase;
                diffphase += (prevphaseov-phaseov)*65536;
                if( adrs != 0 ) wavedata[adrs-1] = (uint)diffphase;
                
                prevphase = phase;
                prevphaseov = phaseov;
                ramadrs++;
            } // for
            wavedata[Apb3.buffsize - 1] = (uint)diffphase;
            return 0;
        }
        
        
        
        // 測定開始ボタンのテキストを変更
        public void btnStartText(string str, Color clr)
        {
            Invoke((DelegateType)delegate
            {
                button_startstop.Text = str;
                button_startstop.BackColor = clr;
            });
        }



        // TimeGraphForm から呼ばれる
        public override void ViewPaintAll()
        {
            UpdateBtnCursorText();
        }


        // カーソルボタンのテキスト変更
        public void UpdateBtnCursorText()
        {
            double r = ((TimeCursor)curlist[0]).time * base.clkperiod;
            double m = ((TimeCursor)curlist[1]).time * base.clkperiod;
            double mr = m - r;
            button_cursorref.Text = "R: " + util.Time2Str(r, base.clkperiod);
            button_cursormes.Text = "M: " + util.Time2Str(m, base.clkperiod);
            if (isDispTime)
            {             // 時間表示
                button_cursormesref.Text = "M-R: " + util.Time2Str(mr, base.clkperiod);
            }
            else
            {                        // 周波数表示
                button_cursormesref.Text = "M-R: " + util.Time2FreqStr(mr, base.clkperiod);
            }
        }


        private void initwave()
        {
            for (int i = 0; i < wavelength; i++)
            {
                wavedata[i] = 0;
            }
        }

        private void FMView_Click(object sender, EventArgs e)
        {
            button_startstop.Focus(); // ComboBox に Focus があると Wheel で動いてしまうので開始ボタンに Focus を移す
        }

    }
}
