﻿//================================================================================
//  
//        FreqResponseGraphCursor.cs      周波数特性グラフのカーソルクラス
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;


namespace APB_3 {
    //-------------------------------------------------------------------------
    // 周波数カーソルクラス
    //-------------------------------------------------------------------------

    public class FreqResponseGraphCursor {
        public FreqResponseGraphWave wave;
        public string name = "";      // カーソル名前
        public double freq = 0;       // カーソル位置の周波数
        public double data = 0;       // カーソル位置のデータ
        public int pos = 0;          // 名前表示位置　0: 下側　1: 上側
        public int namewidth = 0;    // 名前表示幅
        public bool visible = true;   // 表示する・しない
        public Pen pen = new Pen(clr.Cursor, 2);  // カーソル色　太さ=2
        public SolidBrush bgbrush = new SolidBrush(clr.CursorNameBG); // カーソル名前背景色
        public SolidBrush namebrush = new SolidBrush(clr.CursorName); // カーソル名前色

        // コンストラクタ
        public FreqResponseGraphCursor(FreqResponseGraphWave wave) {
            this.wave = wave;
            Normal();
        }

        public FreqResponseGraphCursor(FreqResponseGraphWave wave, double freq) {
            this.wave = wave;
            this.freq = freq;
            Normal();
        }

        public FreqResponseGraphCursor(FreqResponseGraphWave wave, string name) {
            this.wave = wave;
            this.name = name;
            Normal();
        }


        // 標準色にする
        public void Normal() {
            pen.Color = clr.Cursor;    // カーソル色
            pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;    // 
            bgbrush.Color = clr.CursorNameBG;  // カーソル名前背景色
            namebrush.Color = clr.CursorName;      // カーソル名前色
        }


        // ハイライト色にする
        public void Highlight() {
            pen.Color = clr.CursorHighlight;    // カーソル色
            pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;    // 
            bgbrush.Color = clr.CursorHighlightNameBG;  // カーソル名前背景色
            namebrush.Color = clr.CursorHighlightName;      // カーソル名前色
        }


        // 周波数とデータ設定　　　実際にデータのある周波数とその周波数のデータを設定
        public void SetFreqData() {
            SetFreqData(this.freq);
        }

        public void SetFreqData(double freq) {
            try {
                int i = wave.wave.Count-1;
                bool swept = false;
                for( ; i>=0; i-- ) {  // データのある周波数を探す
                    FreqResponseGraphData frd = (FreqResponseGraphData)(wave.wave[i]);
                    if ( frd.freq > freq + 1e-3 ) swept = true;
                    if ( swept && (frd.freq <= freq + 1e-3) ) { // 本当は微小範囲に入っているかの判定が必要！！
                        this.freq = frd.freq;
                        this.data = frd.data;
                        break;
                    }
                }
            }
            catch (Exception e) {
            }
        }

    }
}
