﻿//================================================================================
//  
//        FreqResponseGraphForm.cs      周波数特性グラフ画面
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はできません。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Collections;


namespace APB_3 {
    //-------------------------------------------------------------------------
    // 周波数特性グラフ表示クラス
    //-------------------------------------------------------------------------
    public partial class FreqResponseGraphForm : Form {
        protected bool netanamode;
        protected MainForm mainf;
        private Graphics bgr;                       // ビットマップグラフィックス
        protected int px = 0, py = 0, pw = 500, ph = 400;   // form の中のgraphareaの位置、サイズ
        private int margin = (int)(38 * (MainForm.dpi/96));   // 左右マージン
        private Rectangle grapharea;                // グラフビットマップの大きさ
        public Rectangle wavearea;                  // グラフビットマップの中の波形領域
        private Bitmap bitmap = new Bitmap(1, 1);   // 波形描画用
        public Color FRGScale = clr.FRGScale;       // 色　枠線、スケール
        public Color FRGSubScale = clr.FRGSubScale; // 色　サブスケール
        public Color FRGScaleStrong = clr.FRGScaleStrong; // 色　強調スケール
        public Color FRGName = clr.FRGName;         // 色　文字
        public Color FRGPictBG = clr.FRGPictBG;     // 色　外枠
        public Color FRGGraphBG = clr.FRGGraphBG;   // 色　グラフ領域
        
        public ArrayList wavelist = new ArrayList();// 測定結果波形リスト（複数波形）
        public int level;                           // 測定信号レベル
        
        public string comment = "";                 // コメント
        public DateTime dt = System.DateTime.Now;   // 測定日時
        public bool commentvisible  = false;        // コメント表示するか
        public bool datetimevisible = true;         // 日時表示するか
        public bool conditionvisible= false;        // 測定条件表示するか
        
        public double persistence = 0.8;            // 波形のパーシスタンス
        
        public double fstart = 0;
        public double fstop = Apb3.nyquistfreq * 0.8;
        public double fstep = 1; 
        public double ffull = Apb3.nyquistfreq;     //50MHz
        public ArrayList scale = new ArrayList();   // 縦軸スケール　pos= 0:左側  1:右側
        public int vdiv = 12;           // 縦軸分割数
        public bool logscale = false;   // log 周波数軸スケール
        public bool centerspan = false; // リニア周波数軸のときセンター、スパンでの表示にする

        public bool cursorline = true;  // カーソルライン表示する
        public bool cursorcross = false; // カーソル×表示する
        public bool cursorarrow = true; // カーソル↓表示
        public bool showfreq = true;    // カーソルで周波数表示する
        public bool harmonics = false;  // カーソルの高調波表示をする
        public bool showlevel = true;   // カーソルでのレベル表示をする
        public bool showlevelnearcur = false;   // カーソルでのレベル表示をカーソルの近くにする
        public bool deltaf = true;      // カーソルで周波数差表示表示をする
        
        public ArrayList cursor = new ArrayList();    // カーソル
        public FreqResponseGraphCursor curcursor;     // カレントカーソル
        private bool cursordragging = false;          // カーソルドラッギング中
        
        protected int mousex = 0, mousey = 0;  // pict 座標でのマウス位置
        
        private delegate void DelegateType();  // 別スレッドのコントロールを操作するのに使う
        
        
        public FreqResponseGraphForm() {
            InitializeComponent();
            MouseWheel += new System.Windows.Forms.MouseEventHandler(MouseWheelHandler);
        }
        
        
        // マウスホイール　イベントハンドラー
        protected void MouseWheelHandler(object sender, MouseEventArgs e) {
            if( e.Delta > 0 ) menuExpand();
            else              menuShrink();
        }
        
        
        // マウスボタンが押された
        protected void mouseDown(object sender, MouseEventArgs e) {
            MeasureButtonFocus();       // combobox がマウスホイールイベントを取ったままにしないように
            this.Focus();               // どこをクリックされてもフォーカスがくるように
            mousex = e.X - px;
            mousey = e.Y - py;
            
            if ((Control.ModifierKeys == Keys.None) &&
                (e.Button == MouseButtons.Left) && 
                (isWaveArea() || isCursorArea()) ) {
                // カーソルドラッグ開始
                cursordragging = true;              // カーソルドラッギング中にして
                foreach (FreqResponseGraphCursor cur in cursor) {
                    if (isOnCursor(cur)) {          // カーソルマーク上なら
                        curcursor.Normal();
                        curcursor = cur;            // それをカレントカーソルにする
                        curcursor.Highlight();
                        break;
                    }
                }
                try{
                    curcursor.SetFreqData(x2freq(mousex));// カーソル移動
                    PaintAll();
                }
                catch (Exception ex) {
                    mainf.LogPrint("\nmouseDown: e=" + ex.Message );
                }
            }
        }
        
        
        // 継承先での測定ボタンへのフォーカス呼び出し
        public virtual void MeasureButtonFocus(){}
        
        
        // マウスが移動した　　　カーソルドラッグ
        protected void mouseMove(object sender, MouseEventArgs e) {
            int mx = e.X - px;
            if (mx < wavearea.Left) mx = wavearea.Left;
            if (mx > wavearea.Right) mx = wavearea.Right;
            if ((cursordragging) && (mousex != mx)) {
                mousex = mx;
                mousey = e.Y - py;
                curcursor.SetFreqData(x2freq(mousex));   // カーソル移動
                PaintAll();
            }
        }
        
        
        // マウスボタンが離された　　ドラッグ終了
        protected void mouseUp(object sender, MouseEventArgs e) {
            cursordragging = false;
        }
        
        
        // ウインドウサイズ変更
        protected void sizeChanged(object sender, System.EventArgs e) {
            ReSize();
        }
        
        
        public void ReSize() {
            pw = ClientSize.Width - px;
            ph = ClientSize.Height - py;
            if ((pw <= 0) || (ph <= 0)) return;  // 最小化のときは何もしない
            
            grapharea = new Rectangle(0, 0, pw, ph);  // ビットマップ作成領域
            int wx = margin;              // wavearea 左マージン
//            int wy = margin;              // wavearea 上マージン
            int wy = 34;              // wavearea 上マージン
            int ww = pw - wx - wx;  // 右マージン分も
            int wh = ph - wy - wy;  // 下マージン分も
            for (int i = 0; i < scale.Count; i++)
            {  // スケール表示分のマージンを追加
                if (((FreqResponseGraphScale)scale[i]).visible)
                {
//                    ww -= margin;
                    ww -= 30;
                    if (((FreqResponseGraphScale)scale[i]).pos == 0) wx += 30;
                }
            }
            wavearea = new Rectangle(wx, wy, ww, wh);
            bitmap.Dispose();               // 今までのビットマップを捨てて
            bitmap = new Bitmap(grapharea.Width + 1, grapharea.Height + 1); // 新しく作る
            bgr = Graphics.FromImage(bitmap);
            UpdateBitmap();
            
        }
        
        
        // 再描画イベント
        private void paint(object Sender, PaintEventArgs e) {
            PaintAll();
        }
        
        
        // 測定値をクリップボードにコピー
        protected void copy2clipborad() {
            StringBuilder sb = new StringBuilder();
            FreqResponseGraphWave frw0 = (FreqResponseGraphWave)wavelist[0];
            for ( int i=0; i< frw0.wave.Count; i++) { // データを順番に読む
                sb.Append("\n" + ((FreqResponseGraphData)frw0.wave[i]).freq.ToString());
                foreach (FreqResponseGraphWave frw in wavelist) {
                    sb.Append("\t" + ((FreqResponseGraphData)frw.wave[i]).data.ToString());
                }
            }
            Clipboard.SetDataObject(sb.ToString());
        }
        
        
        // フルスパン表示
        protected void menuFullSpan() {
            fstart = 0;
            fstop = ffull;
            logscale = false;
            UpdateBitmap();
        }
        
        
        // カーソルをセンター周波数へ
        protected void menuCur2Center() {
            logscale = false;
            centerspan = true;
            double fcenter = curcursor.freq;
            double fspan = fstop - fstart;
            fstart = fcenter - fspan / 2;
            fstop = fcenter + fspan / 2;
        }
        
        
        // カーソル間をズーム
        protected void menuZoom() {
            double fmin = 1e12, fmax = 0;
            foreach (FreqResponseGraphCursor cur in cursor) {
                if (cur.freq < fmin) fmin = cur.freq;
                if (cur.freq > fmax) fmax = cur.freq;
            }
            if (fmin < fmax) {
                fstart = fmin;
                fstop = fmax;
                UpdateBitmap();
            }
        }
        
        
        // カーソルを中心に拡大
        protected void menuExpand() {
            if( logscale ) {
                fstart = fstart / 2;
                fstop  = fstop * 2;
                if( fstart < 1            ) fstart = 1;
                if( fstop  > Apb3.clockfreq/2 ) fstop = Apb3.clockfreq/2;
            } else {
                double fcenter = curcursor.freq;
                double fspan = (fstop - fstart)/2+0.1;
                if( Math.Floor(fspan/2+0.5) > Math.Floor(fspan/2) ) fspan += 1; // 奇数→偶数
                fspan = Math.Floor(fspan);
                if (fspan < 100) fspan = 100;
                fstart = fcenter - fspan / 2;
                fstop  = fcenter + fspan / 2;
            }
            UpdateBitmap();
        }
        
        
        // カーソルを中心に縮小
        protected void menuShrink() {
            if( logscale ) {
                fstart = fstart * 2;
                fstop  = fstop / 2;
                if( fstart > curcursor.freq ) fstart /= 2;
                if( fstop  < curcursor.freq ) fstop *= 2;
            } else {
                double fcenter = curcursor.freq;
                double fspan = (fstop - fstart)*2+0.1;
                if( Math.Floor(fspan/2+0.5) > Math.Floor(fspan/2) ) fspan += 1; // 奇数→偶数
                fspan = Math.Floor(fspan);
                if (fspan > Apb3.clockfreq/2) fspan = Apb3.clockfreq/2;
                fstart = fcenter - fspan / 2;
                fstop  = fcenter + fspan / 2;
            }
            UpdateBitmap();
        }
        
        
        // 次のピークへ
        protected void menuNextPeak() {
            curcursor.SetFreqData(PeakSearch(true));
            PaintAll();
        }
        
        
        // リファレンスにする（ノーマライズキャリブレーション）
        protected void menuToRef() {
            foreach (FreqResponseGraphWave frw in wavelist) {
                frw.ToCal();
            }
            UpdateBitmap();
        }
        
        
        // リファレンスをクリアする
        protected void menuClearRef() {
            foreach (FreqResponseGraphWave frw in wavelist) {
                frw.CalClear();
            }
            UpdateBitmap();
        }
        
        
        // オフセット設定
        protected void SetOffset(FreqResponseGraphWave frw, double offset) {
            frw.SetOffset( offset );
            UpdateBitmap();
        }
        
        
        // 
        public void DispLine(double freq, double data) {
            UpdateBitmap();  // とりあえず　本当はちゃんと表示するべき！！
        }
        
        
        // ウインドウ再描画
        public void PaintAll() {
            Graphics g = this.CreateGraphics();
            g.DrawImage(bitmap, px, py);  // grapharea 波形ビットマップを本来の位置にコピー
            DispCursor(g);
            g.Dispose();
            
        }
        
        
        // グラフ表示ビットマップを作り直し再描画
        public void UpdateBitmap() {
            if ((grapharea.Width > 0) && (grapharea.Height > 0)) {     // 範囲が有効か（最小化したときなど）？
                DispFrame();
                DispWave();
                foreach (FreqResponseGraphCursor cur in cursor) {
                    cur.SetFreqData();
                }
                PaintAll();
            }
        }
        
        
        // グラフの枠線、軸　表示
        private void DispFrame() {
            bgr.SetClip(grapharea);

            int i, x1, y1, x2, y2;

            Pen pen = new Pen(FRGScale);            // 枠線、スケール
            Pen penStrong = new Pen(FRGScaleStrong);// 強調スケール
            Brush brush = new SolidBrush(FRGName);  // 数字
            Font Fontnormal = fnt.FRGnormal;
            Font Fontsmall = fnt.FRGsmall;
            Font Fontmid = fnt.FRGmid;

            bgr.FillRectangle(new SolidBrush(FRGPictBG), grapharea);   // 外枠
            bgr.FillRectangle(new SolidBrush(FRGGraphBG), wavearea); // グラフ領域

            // 周波数軸スケール（縦線）
            x1 = wavearea.Left;
            y1 = wavearea.Top;
            x2 = wavearea.Right;
            y2 = wavearea.Bottom;

            if (logscale) {  // LOG 周波数軸表示
                if (fstart <= 0) fstart = 1;
                if (fstart >= fstop) fstop = fstart * 2;
                double lfstart = (fstart <= 0) ? 0 : Math.Log10(fstart);//Math.Log10(fstart);  // LOG開始周波数
                double lfstop = Math.Log10(fstop);  // LOG終了周波数
                double fs = (x2 - x1) / (lfstop - lfstart);
                double f, f1;
                int x;
                int ifb = (int)Math.Floor(lfstart);
                int ife = (int)Math.Floor(lfstop) + 1;
                for (; ifb < ife; ifb++) {
                    f = Math.Pow(10, ifb);
                    for (i = 1; i < 10; i++) {
                        f1 = i * f;
                        if ((f1 >= fstart) && (f1 <= fstop)) {
                            x = (int)((Math.Log10(f1) - lfstart) * fs + x1);
                            if (i == 1) {
                                bgr.DrawLine(pen, x, y1, x, y2 + 5);
                                bgr.DrawString(util.Double2String(f), Fontnormal, brush, x - 6, y2 + 12);
                            }
                            else {
                                bgr.DrawLine(pen, x, y1, x, y2);
                                bgr.DrawString(String.Format("{0}", i), Fontsmall, brush, x - 3, y2 + 4);
                            }
                        }
                    }
                }
            }
            else  {          // リニア周波数軸表示
                double fspan = fstop - fstart;
                double fcenter = (fstop + fstart) / 2;
                double fstep = 1;       // 1Hz
                if (centerspan) {       // 中心周波数とスパン表示
                    while (true) {      // 一番適当な fstep を探す
                        if (fspan / fstep * 5 < 16) { fstep /= 5; break; }
                        if (fspan / fstep * 2 < 16) { fstep /= 2; break; }
                        if (fspan / fstep < 16) break;
                        fstep *= 10;
                    }
                    int n = (int)(((int)(fspan / fstep + 0.001)) / 2) + 1;
                    for (i = 1; i < n; i++) {
                        x1 = wavearea.Left + wavearea.Width / 2 + (int)(wavearea.Width * i * fstep / fspan);
                        bgr.DrawLine(pen, x1, y1, x1, y2);
                        bgr.DrawString("+" + util.Double2String(i * fstep), Fontmid, brush, x1 - 10, y2 + 2);
                        x1 = wavearea.Left + wavearea.Width / 2 - (int)(wavearea.Width * i * fstep / fspan);
                        bgr.DrawLine(pen, x1, y1, x1, y2);
                        bgr.DrawString("-" + util.Double2String(i * fstep), Fontmid, brush, x1 - 10, y2 + 2);
                    }
                    x1 = wavearea.Left + wavearea.Width / 2;
                    bgr.DrawLine(penStrong, x1, y1, x1, y2 + 10);
                    bgr.DrawString(util.Freq2Str(fcenter), Fontnormal, brush, x1 - 20, y2 + 16);
                }
                else {   // 開始・終了周波数表示
                    double fs;
                    while (true) {       // 一番適当な fstep を探す
                        if (fspan / fstep * 5 < 16) { fstep /= 5; break; }
                        if (fspan / fstep * 2 < 16) { fstep /= 2; break; }
                        if (fspan / fstep < 16) break;
                        fstep *= 10;
                    }
                    fs = (int)(fstart / fstep + 1) * fstep;
                    for (; fs < fstop; fs += fstep) {
                        x1 = wavearea.Left + (int)(wavearea.Width * (fs - fstart) / fspan);
                        bgr.DrawLine(pen, x1, y1, x1, y2);
                        bgr.DrawString(util.Double2String_np(fs), Fontmid, brush, x1 - 6, y2 + 2);
                    }
                    x1 = wavearea.Left;
                    bgr.DrawLine(pen, x1, y1, x1, y2 + 10);
                    bgr.DrawString(util.Double2String(fstart), Fontnormal, brush, x1 - 10, y2 + 14);
                    x1 = wavearea.Right;
                    bgr.DrawLine(pen, x1, y1, x1, y2 + 10);
                    string fstopstr = util.Double2String(fstop);
                    bgr.DrawString(fstopstr.PadLeft(12), Fontnormal, brush, x1 - 50, y2 + 14);
                }
            }
            
            // 縦軸スケール（横線）
            x1 = wavearea.Left;
            y1 = wavearea.Top;
            x2 = wavearea.Right;
            y2 = wavearea.Bottom;
            for (i = 0; i < vdiv + 1; i++) {
                y1 = wavearea.Top + wavearea.Height * i / vdiv;
                bgr.DrawLine(pen, x1, y1, x2, y1);
            }
            
            // 縦軸スケール文字
            foreach (FreqResponseGraphScale s in scale) {
                if (s.pos == 0) x1 = grapharea.Left + 12;
                else            x1 = wavearea.Right;
                if (s.visible) {
//                    bgr.DrawString(s.name, Fontnormal, brush, x1 - 4, wavearea.Top - 24);
                    bgr.DrawString(s.name, Fontnormal, brush, x1 - 4, wavearea.Top - 28);
                    double lv = s.top;
                    for (i = 0; i < vdiv + 1; i++) {
                        y1 = wavearea.Top + wavearea.Height * i / vdiv - 10;
                        string lvstr = util.Double2String_0(lv);
                        bgr.DrawString(lvstr.PadLeft(8), Fontnormal, brush, x1, y1);
                        lv -= s.perdiv;
                    }
                }
            }
            
            string str = "";
            // コメントなどの表示
            if( commentvisible ) str += comment+"\n";
            if( datetimevisible ) str += dt.ToString()+"\n";
            if( conditionvisible ) str += comment;
            bgr.DrawString( str, fnt.FRGcursor, brush, wavearea.Left+10, wavearea.Top+4);
            
            // 信号レベル表示
            if( level >= 6 ) {
              if( level == 7 ) brush = new SolidBrush(clr.FRGStrong);
              bgr.DrawString("OVER", Fontnormal, brush, (wavearea.Left+wavearea.Right)/2-50, wavearea.Top);
            }
            
            pen.Dispose();
            penStrong.Dispose();
            brush.Dispose();
        }
        
        
        // 波形表示
        private void DispWave() {
            try {
                bgr.SetClip(wavearea);
                
                const int fmargin=50;   // 画面に測定点が少ないとき画面の両端で乱れるのを防ぐマージン
                
                ArrayList fs = new ArrayList();         // x軸の各位置での周波数を入れておく
                if (logscale)  {
                    double lfstart = (fstart <= 0) ? 0 : Math.Log10(fstart);  // LOG開始周波数
                    double lfstop = Math.Log10(fstop);    // LOG終了周波数
                    for (int i = -fmargin; i < wavearea.Width+1+fmargin; i++) {
                        fs.Add(Math.Pow(10, i * (lfstop - lfstart) / wavearea.Width + lfstart));
                    }
                }
                else { // リニアスケール
                    for (int i = -fmargin; i < wavearea.Width+1+fmargin; i++) {
                        fs.Add(i * (fstop - fstart) / wavearea.Width + fstart);
                    }
                }
                fs.Add(1e12); // センチネルとして十分高い周波数を設定しておく

                foreach (FreqResponseGraphWave frw in wavelist) { // すべての波形データ
                    if( !frw.visible ) continue;
                    if (frw.wave.Count == 0) break;
                    FreqResponseGraphScale s = ((FreqResponseGraphScale)scale[frw.scaleno]);
                    Pen pen = (Pen)(frw.pen.Clone());
                    Color c1 = pen.Color;
                    double ratio = Math.Pow(persistence, frw.GetNsweep()-1);
                    pen.Color = IntermediateColor( c1, FRGGraphBG, ratio );
                    int x1 = 0;
                    int x2 = 0;
                    int y1 = 0;
                    int y2;
                    int i = 0;
                    bool first = true;
                    foreach (FreqResponseGraphData frd in frw.wave) { // データを順番に表示
                        double data = frd.data;
                        y2 = (int)(wavearea.Height * (1 - (s.perdiv * vdiv - s.top + data) / (s.perdiv * vdiv)));
                        if (y2 < 0) y2 = -5;
                        int i1 = (i == 0) ? 0: i - 1;
                        if (frd.freq < (double)fs[i1]) i = 0; // 周波数が低くなったら最初から探す
                        for (; i < wavearea.Width+2+2*fmargin; i++)  {    // 周波数に対応する x を探す
                            if (frd.freq <= (double)fs[i]) {
                                x2 = i-fmargin;
                                break;
                            }
                        }
                        if ( !first ) {       // 最初の1回は描画しない
                            if( x1 <= x2 ) {  // 周波数が増加していたら表示
                                if( ratio > 0.05 ) // 色が充分濃いときだけ表示
                                    bgr.DrawLine(pen, x1 + wavearea.Left - 1, y1 + wavearea.Top, x2 + wavearea.Left - 1, y2 + wavearea.Top);
                            } else {          // 周波数が逆転したら次の色（濃くする）
                                ratio = ratio/persistence;
                                pen.Color = IntermediateColor( c1, FRGGraphBG, ratio );
                            }
                        }
                        x1 = x2;
                        y1 = y2;
                        first = false;
                    }
                }
                fs.Clear();
            }
            catch (Exception e) {
                mainf.LogPrint("\nFreqResponseGraphForm.DispWave: e=" + e.Message );
            }
        }
        
        
        // カーソル表示　　　　　　表示は実際の画面なので座標に注意
        private void DispCursor(Graphics g) {
            try {
                g.SetClip(new Rectangle(px, py, pw, ph));
                foreach (FreqResponseGraphCursor cur in cursor) {
                    double freq = cur.freq;
                    if( freq <= 0 ) freq = 0.001;
                    Pen pen = (Pen)(cur.pen.Clone()); //cur.pen;
                    SolidBrush bgbrush = cur.bgbrush;
                    SolidBrush namebrush = cur.namebrush;
                    string name = cur.name;
                    int x1 = freq2x(freq) + px;
                    int y1 = wavearea.Top + py;
                    int y2 = y1 + wavearea.Height;
                    y1 += (cur.pos % 2 == 0) ? -1 : -15;    // カーソルの値表示位置
                    FreqResponseGraphScale s = ((FreqResponseGraphScale)scale[cur.wave.scaleno]);
                    int yd = (int)(wavearea.Height * (1 - (s.perdiv * vdiv - s.top + cur.data) / (s.perdiv * vdiv))) + wavearea.Top + py;
                    if (yd < 0) yd = 5;
                    if ((cur.visible) && ((int)(freq + 0.5) >= (int)(fstart + 0.5)) && ((int)(freq + 0.5) <= (int)(fstop + 0.5))) {
                        if (cursorline) {        // カーソルライン表示
                            g.DrawLine(pen, x1, y1, x1, y2);
                        }
                        pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Solid;    // カーソルマークは実線にする
                        if (cursorcross) {       // カーソル×表示
                            g.DrawLine(pen, x1-5, yd-5, x1+5, yd+5);
                            g.DrawLine(pen, x1-5, yd+5, x1+5, yd-5);
                        } else if( cursorarrow ) { // カーソル↓表示
                            g.DrawLine(pen, x1  , yd  , x1-5, yd-10);
                            g.DrawLine(pen, x1-1, yd  , x1+4, yd-10);
                            g.DrawLine(pen, x1  , yd  , x1  , yd-13);
                        }
                        string strfreq;
                        if (showfreq) {        // 周波数表示
                            if (deltaf && (cur != curcursor)) {
                                strfreq = "⊿" + util.Freq2Str(freq - curcursor.freq);
                            }
                            else {
                                strfreq = cur.name + util.Freq2Str(freq);
                            }
                        }
                        else {
                            strfreq = cur.name;
                        }
                        int w = 7 * strfreq.Length;
                        cur.namewidth = w;
                        g.FillRectangle(bgbrush, x1 - w / 2, y1 - 13, w, 13);
                        g.DrawString(strfreq, fnt.FRGcursor, namebrush, x1 - w / 2, y1 - 14);
                        
                        string strlevel;
                        if (showlevel) {     // レベル表示
                            if (deltaf && (cur != curcursor)) {
                                strlevel = "⊿" + util.Double2String_00(cur.data - curcursor.data);
                            } else {
                                strlevel = util.Double2String_00(cur.data);
                            }
                            if( showlevelnearcur ) {
                                g.DrawString(strlevel + ((FreqResponseGraphScale)scale[cur.wave.scaleno]).unit, fnt.FRGcursor, bgbrush, x1+2, y1+15);
                            } else {
                                g.DrawString(strfreq, fnt.FRGcursor, bgbrush, wavearea.Right-200, y1+18);
                                g.DrawString(strlevel + ((FreqResponseGraphScale)scale[cur.wave.scaleno]).unit, fnt.FRGcursor, bgbrush, wavearea.Right-90, y1+18);
                            }
                        }
                        if (harmonics) {     // 高調波カーソル表示
                            Pen pen1 = (Pen)(pen.Clone());
                            pen1.Width = 1;
                            freq = cur.freq;
                            y1 = wavearea.Top + py;
                            y2 = y1 + wavearea.Height;
                            for (int i = 2; i <= 10; i++) {  // 高調波１０次まで
                                if (i * freq > fstop) break;
                                x1 = freq2x(i * freq) + px;
                                g.DrawLine(pen1, x1, y1, x1, y2);
                                if (cur == curcursor)
                                    g.DrawString(i.ToString(), fnt.FRGcursor, bgbrush, x1 - 4, y1 - 15);
                            }
                        }
                    }
                }
            }
            catch (Exception e) {
                mainf.LogPrint("\nFreqResponseGraphForm.DispCursor: e=" + e.Message );
            }
        }
        
        
        // 二つの色の中間色を作成 （ratio が 1 に近いほど C1 に近く 0 に近いほど C2 に近い色を返す）
        public Color IntermediateColor( Color c1, Color c2, double ratio ) {
            if( ratio >= 1 ) ratio = 1;
            int r, g, b;
            int r1 = c1.R;
            int g1 = c1.G;
            int b1 = c1.B;
            int r2 = c2.R;
            int g2 = c2.G;
            int b2 = c2.B;
            
            r = (int)(r1*ratio + r2*(1-ratio));
            g = (int)(g1*ratio + g2*(1-ratio));
            b = (int)(b1*ratio + b2*(1-ratio));
            
            return Color.FromArgb( r, g, b );
        }
        
        
        // 別の色を作成 （ C1 と輝度が同じで違う色を返す）
        static int ncolor=0;
        public Color DifferentColor( Color c1 ) {
            ncolor++;
            int r, g, b;
            double y = (0.3*c1.R + 0.59*c1.G + 0.11*c1.B)/91;
            
            int r1 = 13 * (ncolor%7);
            int g1 = 7 * (ncolor%13);
            int b1 = 91-r1-g1;
            if( b1 < 0 ) b1=0;
            
            r = (int)(y/0.3*r1);
            g = (int)(y/0.59*g1);
            b = (int)(y/0.11*b1);
            
            if( r>255 ) r=255;
            if( g>255 ) g=255;
            if( b>255 ) b=255;
mainf.LogPrint("\nDifferentColor: " + (y*91).ToString() + " " + r1.ToString() + " " + g1.ToString() + " " + b1.ToString() + " " + r.ToString() + " " + g.ToString() + " " + b.ToString() );
            
            return Color.FromArgb( r, g, b );
        }
        
        
        // ピーク周波数を探す     next==  false:低い周波数へ　true:高い周波数へ
        public double PeakSearch(bool next) {
            double pdata = 99999, pfreq = 0, pdiff = -99999, diff = 0, intdiff = 0;
            foreach (FreqResponseGraphData frd in curcursor.wave.wave) {
                diff = (frd.data - pdata);  // 微分
                if (pdiff > -2) intdiff += pdiff;
                else if (pdiff < -2) intdiff = 0;
                if ((pdiff > 0) && (diff < 0) && (intdiff > 5)) {
                    if (intdiff > 10) {
                        if (pfreq > curcursor.freq) return pfreq;
                    }
                }
                pdata = frd.data;
                pfreq = frd.freq;
                pdiff = diff;
            }
            return curcursor.freq;
        }
        
        
        // グラフのｘ位置から周波数へ変換
        public double x2freq(int x) {
            double freq=0;
            if (logscale) {  // LOG 周波数軸
                double lfstart = (fstart <= 0) ? 0 : Math.Log10(fstart);//Math.Log10(fstart);  // LOG開始周波数
                double lfstop = Math.Log10(fstop);  // LOG終了周波数
                double lfspan = (lfstop - lfstart);
                freq = Math.Pow(10, lfspan * (x - wavearea.Left) / wavearea.Width + lfstart);
            } else {
                double fspan = fstop - fstart;
                freq = fspan * (x - wavearea.Left) / wavearea.Width + fstart;
            }
            try {
                // 実際にデータのある周波数にする
                foreach (FreqResponseGraphData frd in curcursor.wave.wave) {
                    if (freq <= frd.freq + 1e-10) { // 本当は微小範囲に入っているかの判定が必要！！
                        freq = frd.freq;
                        //return freq;
                        break;
                    }
                }
            }
            catch (Exception ex) {
            }
            return freq;
        }
        
        
        // 周波数からグラフのｘ位置へ変換
        public int freq2x(double freq) {
            if (logscale) {  // LOG 周波数軸
                double lfreq = Math.Log10(freq);
                double lfstart = (fstart <= 0) ? 0 : Math.Log10(fstart);//Math.Log10(fstart);  // LOG開始周波数
                double lfstop = Math.Log10(fstop);  // LOG終了周波数
                double lfspan = (lfstop - lfstart);
                int x = (int)((lfreq - lfstart) / lfspan * wavearea.Width + 0.5) + wavearea.Left;
                return x;
            }
            else {
                double fspan = fstop - fstart;
                int x = (int)((freq - fstart) / fspan * wavearea.Width + 0.5) + wavearea.Left;
                return x;
            }
        }
        
        
        // マウスがカーソルの上か
        public bool isOnCursor(FreqResponseGraphCursor cur) {
            int ylimit = wavearea.Top;
            if (mousey > wavearea.Top + wavearea.Height) return false;
            double freq = cur.freq;
            int xf = freq2x(freq);
            if (mousey >= ylimit) {  // カーソル線部分
                if ((mousex < xf + 7) && (mousex > xf - 7)) return true;
            }
            else {                       // カーソル名表示部分
                if ((cur.pos == 0) && (mousey >= ylimit - 15)) return true;
//                if ((cur.pos == 1) && (mousey >= ylimit - 30) && (mousey < ylimit - 15)) return true;
                if ((cur.pos == 1) && (mousey < ylimit - 15)) return true;
            }
            return false;
        }
        
        
        // マウスがグラフ表示部分の上か
        public bool isGraphArea() {
            return ((mousex >= 0) && (mousey >= 0));
        }
        
        
        // マウスが波形表示部分の上か
        public bool isWaveArea() {
            if (isGraphArea())
            {
                if (isFreqArea()  ) return false;
                if (isLeftArea()  ) return false;
                if (isRightArea() ) return false;
                if (isCursorArea()) return false;
                return true;
            }
            return false;
        }
        
        
        // マウスが周波数表示部分の上か
        public bool isFreqArea() {
            if (isGraphArea()) return (mousey > (wavearea.Top + wavearea.Height));
            return false;
        }
        
        
        // マウスが左軸表示部分の上か
        public bool isLeftArea() {
            if (isGraphArea()) return ((mousey < (wavearea.Top + wavearea.Height)) && (mousex < wavearea.Left));
            return false;
        }
        
        
        // マウスが右軸表示部分の上か
        public bool isRightArea() {
            if (isGraphArea()) return ((mousey < (wavearea.Top + wavearea.Height)) && (mousex > wavearea.Right));
            return false;
        }
        
        
        // マウスがカーソル表示部分の上か
        public bool isCursorArea() {
            if (isGraphArea()) return (mousey < wavearea.Top);
            return false;
        }
        
        
    }
    
}
