﻿//================================================================================
//  
//        TimeGraphForm.cs           タイムスケールを持つグラフの共通クラス
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど
//  商業的利用はおやめください。
//


using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO.Ports;
using System.Linq;
using System.Threading;
using System.Text;
using System.Windows.Forms;

namespace APB_3
{
    //-------------------------------------------------------------------------
    // タイムグラフ（時間軸を持つグラフ）フォーム
    //-------------------------------------------------------------------------
    public partial class TimeGraphForm : Form
    {

        public uint[] wavedata = new uint[Apb3.buffsize / 4 * 4];  // 波形データ 8bit モード時は 4 倍になるので
        public int wavelength = Apb3.buffsize / 2;
        public double clkperiod = 2 / Apb3.clockfreq;  // サンプリングタイム

        public MainForm mainf;
        public int gx = 130, gy = 32;             // グラフパネルの位置
        public int gw = 300, gh = 200;            // グラフパネルの大きさ
        protected int cmenuWaveno = 0;          // コンテキストメニューの波形番号

        protected MenuItem menuWavecolor;       // 　波形色
        protected MenuItem menuDel;             // 　削除
        protected MenuItem menuMagfull;         // 　タイムスケールを　フルスパン⇔直前の拡大
        protected MenuItem menuGotoref;         // 　refカーソルへ移動
        protected MenuItem menuGotomes;         // 　mesカーソルへ移動

        protected TimeScale timescale;          // タイムスケール
        protected ArrayList wavelist = new ArrayList(); // 波形ウインドウのリスト
        protected ArrayList curlist = new ArrayList(); // カーソルのリスト
        public TimeCursor curcursor;            // 現ハイライトカーソル
        public int triggerpoint = -10000;       // トリガーポイント

        protected int mousex = 0;
        protected int mousey = 0;

        private bool timescaledragging = false; // タイムスケール上でドラッグ中  
        private bool cursordragging = false;    // カーソルドラッグ中  
        private bool wavewidthdragging = false; // 波形幅ドラッグ中
        private bool dsoposdragging = false;    // DSO垂直軸表示位置ドラッグ中
        private bool waveposdragging = false;   // 波形位置ドラッグ中
        private TimeGraphWave dragwave;         // ドラッグ波形
        private int dragwaveno = 0;             // ドラッグ元波形番号
        private int dragwavey = -1;             // ドラッグ波形挿入位置
        private int dragmousey;                 // ドラッグ開始時マウスｙ

        private int wavebegin = 0;              // 波形表示開始番号
        protected int waven = 0;                // 波形表示個数 =wavelist.Count
        public int wavex = (int)(100 * (MainForm.dpi/96)); // 波形表示部分始まり（０～ここまでは波形名表示）
        public int wavey = 30;                  // 波形表示の開始y位置（グラフ内）＝タイムスケールの幅
        public int wavey1 = 0;                  // 波形表示の最後y位置
        public int wavey2 = 0;                  // 波形の最後y位置

        private double[] multitable = new double[] { 0.02, 0.05, 0.1, 0.2, 0.5, 1, 2, 5, 10, 20, 50 };   //拡大率 1/50～50
        public int imulti = 5;                  // multi = 1 になる初期値
        private int imultifull = -1;            // タイムスケールフルのときの拡大率
        public double multi = 1;                // 拡大率（１サンプルが画面上何ピクセルになるか）
        public int sttime = 0;                  // 波形表示開始　サンプリング時間(0-2047)
        public int previmulti = 4;              // 直前の拡大率
        public int prevsttime = 0;              // 直前の波形表示開始
        private int iscale = 5;                 // div 設定のどれを使っているか
        public int divpix = 100;                // １divのピクセル数
        public int subdivpix = 20;              // １nsubdivのピクセル数
        public int nsubdiv = 4;                 //  nsubdiv分割数
        public int divtime = 100;               // １divの時間
        public string timeunit = "";            // １divの時間単位
        public string subdiv = "";              // １subdivの時間単位
        public int Highlight = -1;              // ハイライトしている波形番号


        //  コンストラクタ
        public TimeGraphForm()
        {
            timescale = new TimeScale(this, wavey);   // InitializeComponent より先にインスタンス生成しないとヌルオブジェクトになる？ why?
            InitializeComponent();
            
            AddHScrollBar();
            AddVScrollBar();
            MouseWheel += new System.Windows.Forms.MouseEventHandler(MouseWheelHandler);
        }



        // hScrollBar
        protected void AddHScrollBar()
        {
            hScrollBar1.Location = new Point(0, gh - 18);
            hScrollBar1.Size = new Size(gw - 18, 18);    // vscrollbar の幅分
            hScrollBar1.Minimum = 0;
            hScrollBar1.Maximum = (int)wavelength;
            hScrollBar1.Scroll += new ScrollEventHandler(hScrollBarScrollHandler);
        }


        // vScrollBar
        protected void AddVScrollBar()
        {
            vScrollBar1.Location = new Point(gw - 18, 0);
            vScrollBar1.Size = new Size(18, gh - 18); // hscrollbar の幅分
            vScrollBar1.Minimum = wavey;
            vScrollBar1.Maximum = wavey;              // 波形表示の最後y位置
            vScrollBar1.SmallChange = 20;             // 波形表示の最後y位置
            vScrollBar1.LargeChange = gh - 18 - 20;        // 波形表示の最後y位置
            vScrollBar1.Scroll += new ScrollEventHandler(vScrollBarScrollHandler);
        }


        // ウインドウ再描画
        protected void graphPaint(object Sender, PaintEventArgs e)
        {
            PaintAll();
        }


        // 水平スクロールバー　スクロールイベント
        protected void hScrollBarScrollHandler(Object sender, ScrollEventArgs e)
        {
            switch (e.Type)
            {
                case ScrollEventType.LargeDecrement:
                case ScrollEventType.LargeIncrement:
                case ScrollEventType.SmallDecrement:
                case ScrollEventType.SmallIncrement:
                case ScrollEventType.ThumbTrack:
                    setStartTime(e.NewValue - sttime);
                    UpdatePaint();
                    break;
            }
        }


        // 垂直スクロールバー　スクロールイベント
        protected void vScrollBarScrollHandler(Object sender, ScrollEventArgs e)
        {
            //mainf.debugprint( String.Format( "\rvScrollBarScrollHandler   e.Type:{0} ",e.Type ) );
            int oldwavebegin = wavebegin;               // 波形表示開始番号
            switch (e.Type)
            {
                case ScrollEventType.SmallDecrement: wavebegin--; break;
                case ScrollEventType.SmallIncrement: wavebegin++; break;
                case ScrollEventType.LargeDecrement:
                case ScrollEventType.LargeIncrement:
                case ScrollEventType.ThumbTrack:
                    int i, y1 = wavey;
                    for (i = 0; i < waven; i++)
                    {      // つまみ位置に対応する波形を探す
                        y1 += ((TimeGraphWave)(wavelist[i])).h;
                        if (y1 > e.NewValue)
                        {
                            break;
                        }
                    }
                    wavebegin = i;
                    if (e.NewValue + vScrollBar1.LargeChange > vScrollBar1.Maximum) wavebegin++;
                    break;
            }
            switch (e.Type)
            {
                case ScrollEventType.LargeDecrement:
                case ScrollEventType.LargeIncrement:
                case ScrollEventType.SmallDecrement:
                case ScrollEventType.SmallIncrement:
                case ScrollEventType.ThumbTrack:
                    wavebegin = (wavebegin < 0) ? 0 : wavebegin;
                    wavebegin = (wavebegin >= waven) ? waven - 1 : wavebegin;
                    if (oldwavebegin != wavebegin) PaintAll();
                    break;
            }
        }


        // マウスホイール　イベントハンドラー
        protected void MouseWheelHandler(object sender, MouseEventArgs e)
        {
            mousex = e.X;
            mousey = e.Y;
            if ((mousex < gx) || (mousey < gy)) return;      // グラフ領域以外ならおしまい
            
            int waveno = ClickedWave(mousex, mousey);
            if ( (waveno >= 0) && (mousex < gx + wavex) )
            {
                TimeGraphWave wave = (TimeGraphWave)wavelist[waveno];
                if (wave.radix >= 17)  // アナログ表示
                {   // DSO で波形名前部分なら垂直軸感度変更
                    int index = wave.dso_perdivindex;
                    if (e.Delta >= 120) index++;
                    if (e.Delta <= -120) index--;
                    if (index >= wave.dso_perdivtbl.Length) index--;
                    if (index < 0) index = 0;
                    wave.dso_perdivindex = index;
                    UpdatePaint();
                    return;
                }

            }

            if (mousex > gx + wavex)
            {   // 時間軸拡大縮小
                int newmulti = imulti + e.Delta / 120;
                MultiCursorFix(newmulti);     // カーソル位置固定のズーム
                UpdatePaint();
            }
        }


        // マウスが押された
        protected void graphMouseDown(object sender, MouseEventArgs e)
        {
            this.Focus();                       // どこをクリックされてもフォーカスがくるように
            mousex = e.X;
            mousey = e.Y;
            
            if ((mousex < gx) || (mousey < gy)) return;      // グラフ領域以外ならおしまい
            
            int waveno = ClickedWave(mousex, mousey);
            int mousextime = (int)((mousex - gx - wavex) / multi + 0.5) + sttime;
            
            if (mousey < gy + wavey)
            {   // タイムスケール上
                if (e.Button == MouseButtons.Left)
                {       // 左クリックなら
                    foreach (TimeCursor cur in curlist)
                    {
                        if (cur.isOnCursor(mousex, mousey))
                        { // カーソルマーク上なら
                            curcursor = cur;                      // カレントカーソルにし、
                            curcursor.time = mousextime;          // カーソル移動
                            CursorHighlight(curcursor);
                            cursordragging = true;                  // カーソルドラッギング中
                            SetCursorInArea();
                            UpdatePaint();
                            return;
                        }
                    }
                }
            }
            else if (waveno >= 0)
            {
                if (mousex > gx + wavex)
                { // 波形上
                    if (Math.Abs(mousex - curcursor.cursorx) < 10)
                    { // 現カーソルとクリックが近く
                        if (Highlight == waveno)
                        {                 // いままでのハイライトと同じ波形上なら
                        }
                        else
                        {
                            WaveNormal();                           // 今までのハイライトを戻す
                            ((TimeGraphWave)wavelist[waveno]).Highlight();   // 新しい位置をハイライト
                            Highlight = waveno;
                            curcursor.time = mousextime;            // クリック位置にカーソル
                            cursordragging = true;

                        }
                    }
                    else if (e.Button == MouseButtons.Left)
                    {  // いままでのハイライトと違う波形の左クリック
                        WaveNormal();                             // 今までのハイライトを戻す
                        ((TimeGraphWave)wavelist[waveno]).Highlight();     // 新しい位置をハイライト
                        Highlight = waveno;
                        foreach (TimeCursor cur in curlist)
                        {
                            if (cur.isOnCursor(mousex, mousey))
                            {// カーソルマーク上なら
                                curcursor = cur;                      // カレントカーソルにし、
                                CursorHighlight(curcursor);
                            }
                        }
                        curcursor.time = mousextime;            // クリック位置にカーソル
                        cursordragging = true;
                    }
                }
                else
                {  // 波形名前上
                    if (e.Button == MouseButtons.Left)
                    {     // 左クリックで波形ドラッグ
                        dragwave = ((TimeGraphWave)wavelist[waveno]);
                        dragwaveno = waveno;                    // ドラッグ元波形番号
                        WaveNormal();                           // 今までのハイライトを戻す
                        dragwave.Highlight();                   // ドラッグする波形をハイライト
                        Highlight = waveno;
                        UpdatePaint();
                        if (mousey > dragwave.y + dragwave.h - 8)
                        {               // 波形幅ドラッグ
                            wavewidthdragging = true;
                            this.Cursor = Cursors.HSplit;
                        }
                        else if( (dragwave.radix >= 17) && ( mousex < gx + wavex ) )
                        {               // アナログ表示のときは垂直軸表示位置変更
                            dsoposdragging = true;
                            dragmousey = mousey;
                        }
                        else
                        {               // 波形位置ドラッグ
                            waveposdragging = true;
                            showWavePosCursor(wavex, dragwave.y);
                        }
                        return;
                        
                    }
                }
            }
            else
            {          // 波形のない部分
                WaveNormal();                     // 今までのハイライトを戻す
            }
            SetCursorInArea();
            UpdatePaint();
        }


        // マウスが移動した
        protected void graphMouseMove(object sender, MouseEventArgs e)
        {
            mousex = e.X;
            mousey = e.Y;
            if (waveposdragging)
            {     // 波形の位置変更
                dispWavePosDragging();
            }
            else if (dsoposdragging)
            {     // DSO垂直軸表示位置変更
                dispDsoPosDragging();
            }
            else if (wavewidthdragging)
            {     // 波形の幅変更
                dispWaveWidthDragging();
            }
            else
            {
                int cursortime = (int)((mousex - gx - wavex) / multi + 0.5) + sttime; // マウス位置のカーソル時間
                if ((cursordragging || timescaledragging) && cursortime != curcursor.time)
                {
                    curcursor.time = cursortime;   // カーソル移動
                    SetCursorInArea();
                    PaintAll();
                }
            }
            return;
        }


        // マウスボタンが離された
        protected void graphMouseUp(object sender, MouseEventArgs e)
        {
            mousex = e.X;
            mousey = e.Y;
            int waveno = ClickedWave(mousex, mousey); //
            if (wavewidthdragging)
            {
                wavewidthdragging = false;
                this.Cursor = Cursors.Default;
                PaintAll();
                return;
            }
            if (dsoposdragging)
            {
                dsoposdragging = false;
                PaintAll();
                return;
            }
            if (waveposdragging)
            {
                waveposdragging = false;
                if (waveno != dragwaveno)
                {
                    if (mousex < gx + wavex)
                    {                  // 波形名前部分
                        int wavenoto = waveno;
                        if (waveno < 0) wavenoto = (mousey > gy + wavey1) ? waven : 0;
                        WaveMove(dragwaveno, wavenoto);
                    }
                }
                PaintAll();
                return;
            }
            if (cursordragging)
            {
                cursordragging = false;
                return;
            }
            if (timescaledragging)
            {
                timescaledragging = false;
                return;
            }
        }


        private void dispWavePosDragging()
        {
            int waveno = ClickedWave(mousex, mousey);    // 
            int y1;
            if (mousey > wavey1)
            {
                y1 = ((TimeGraphWave)(wavelist[waven - 1])).y + ((TimeGraphWave)(wavelist[waven - 1])).h;
            }
            else
            {
                if (waveno < 0) waveno = 0;
                y1 = ((TimeGraphWave)(wavelist[waveno])).y;
            }
            if (y1 != dragwavey)
            {
                PaintAll();
                showWavePosCursor(wavex, y1);
                dragwavey = y1;
            }
        }


        // 波形間に位置ドラッグカーソルを表示する
        private void showWavePosCursor(int x, int y)
        {
            Graphics gr = this.CreateGraphics();
            Pen curpen = new Pen(clr.WavePosDrag, 2);
            gr.DrawLine(curpen, gx + 1, y - 5, gx + 1, y + 5);
            gr.DrawLine(curpen, gx + 0, y, gx + x, y);
            gr.DrawLine(curpen, gx + x - 1, y - 5, gx + x - 1, y + 5);
            gr.Dispose();
        }
        
        
        
        private void dispDsoPosDragging()
        {
            int y = mousey;
            if( y != dragmousey ) {
                int dy = y - dragmousey;
                dragmousey = y;
                dragwave.dso_top += dragwave.dso_perdiv * dragwave.dso_ndiv / dragwave.h * dy;
                dragwave.Update();
                PaintAll();
            }
        }
        
        
        
        private void dispWaveWidthDragging()
        {
            int ymin = dragwave.y;
            int y1 = mousey - ymin;
            int gh;
            gh = (y1 < 20) ? (y1) / 2 * 2 : (y1 < 50) ? (y1) / 5 * 5 : (y1) / 16 * 16;     // 増減幅を幅によって変える
            if( dragwave.radix >= 17 )  // アナログ表示
            {
                if (gh < 32 ) gh = 32;    // 最低幅は３２
                if (gh > 1000) gh = 1000;  // 最大幅は１０００
            }
            else
            {
                if (gh < 12 ) gh = 12;   // 最低幅は１２
                if (gh > 256) gh = 256;  // 最大幅は２５６
            }
            if (dragwave.h != gh)
            {
                dragwave.h = gh;
                dragwave.Update();
                PaintAll();
            }
        }


        // ウィンドウサイズ変更
        protected void graphSizeChanged(object sender, System.EventArgs e)
        {
            ReSize();
        }


        public void ReSize()
        {
            //            mainf.debugprint("\rgraphSizeChanged ");
            gw = ClientSize.Width - gx;
            gh = ClientSize.Height - gy;
            if (gw <= 0 || gh <= 0) return;
            hScrollBar1.Location = new Point(gx, gy + gh - 18);
            hScrollBar1.Size = new Size(gw - 18, 18);
            vScrollBar1.Location = new Point(gx + gw - 18, gy);
            vScrollBar1.Size = new Size(18, gh - 18);
            vScrollBar1.LargeChange = gh - 18 - 20;
            UpdateAll();
        }


        // ウインドウサイズ変更
        protected void sizechange(int x, int y, int gw, int gh)
        {
            Location = new Point(x, y);
            Size = new Size(gw, gh);
        }


        // 表示の始まり時間（サンプリング時間）を dtime ずらす
        private void setStartTime(int dtime)
        {
            sttime += dtime;
            if (sttime < 0) sttime = 0;
            if (sttime >= wavelength - 1) sttime -= dtime;
        }


        // 表示ビットマップをすべて作り直してから再描画
        public void UpdatePaint()
        {
            UpdateAll();
            PaintAll();
        }


        // 波形ビットマップとタイムスケールビットマップをすべて作り直す
        public void UpdateAll()
        {
            foreach (TimeGraphWave w in wavelist) w.Update();
            Multi(imulti);
            timescale.Update();
            hScrollBar1.Maximum = wavelength;
            hScrollBar1.Value = (sttime > 0) ? sttime : 0;
            hScrollBar1.SmallChange = (int)(divpix / multi + 0.5);
            int dw = (int)((gw - wavex - 18) / multi);
            if (dw < 1) dw = 1;
            hScrollBar1.LargeChange = dw;
        }


        // ウインドウ再描画
        public void PaintAll()
        {
            Graphics gr = this.CreateGraphics();
            timescale.Disp(gr);

            int y1 = gy + wavey;
            for (int i = wavebegin; i < waven; i++)
            {
                ((TimeGraphWave)(wavelist[i])).Disp(gr, gx, y1);
                y1 += ((TimeGraphWave)wavelist[i]).h;
            }
            // 波形表示していないところを塗りつぶす
            gr.FillRectangle(new SolidBrush(clr.GraphBG), gx, y1, gw - 18, gh);
            wavey1 = y1;

            // カーソル表示
            foreach (TimeCursor cur in curlist) cur.Disp(gr, cur.time);

            int y2 = wavey;
            for (int i = 0; i < wavebegin; i++) y2 += ((TimeGraphWave)wavelist[i]).h;
            vScrollBar1.Value = y2;
            ViewPaintAll();
            gr.Dispose();
        }


        // 継承先での再描画呼び出し
        public virtual void ViewPaintAll() { }


        // カーソル位置を固定して拡大率を変更し実拡大率とそれに伴う設定を変更する
        public void MultiCursorFix(int imulti)
        {
            double oldmulti = multi;
            double olddifftime = (curcursor.time - sttime) * oldmulti;
            Multi(imulti);
            if (olddifftime > 0 && olddifftime < gw - 18)
            {   // カーソルが画面内ならカーソル位置固定
                int difftime = (int)(olddifftime / multi);
                sttime = curcursor.time - difftime;
            }
        }


        // 拡大率を変更し実拡大率とそれに伴う設定を変更する
        public void Multi(int imulti)
        {
            int i;
            if (imulti < 0) imulti = 0;
            if (imulti > multitable.Length - 1) imulti = multitable.Length - 1;
            this.imulti = imulti;
            multi = multitable[imulti];
            double pixtime = clkperiod / multi;  // １ピクセルあたりの実時間
            for (i = 0; i < TimeScale.scaletbl.Length - 1; i++)
            {
                if (pixtime * 50 < TimeScale.scaletbl[i].time) break;    // １div >= 50 ピクセル
            }
            iscale = i;
            divpix = (int)(TimeScale.scaletbl[i].time / pixtime + 0.5);  // １div のピクセル数
            nsubdiv = TimeScale.scaletbl[i].nsubdiv;     // nsubdiv 分割数
            divtime = TimeScale.scaletbl[i].div;          // １divの時間
            timeunit = TimeScale.scaletbl[i].unit;      // スケールの時間単位
            subdiv = TimeScale.scaletbl[i].subdiv;      // xx s/div 表示
        }


        // カーソルを画面センターにもってくる
        public void moveCursorCenter(TimeCursor cur)
        {
            Graphics gr = this.CreateGraphics();
            cur.Disp(gr, (int)(sttime + (gw - wavex - 18) / multi / 2));
            SetCursorInArea();
            gr.Dispose();
            UpdatePaint();
        }


        // ２つの時間の間をズームして表示する
        public void ZoomBetween(int time0, int time1)
        {
            int diff = Math.Abs(time0 - time1);
            double m = (double)(gw - wavex - 18) / diff;// 横幅いっぱいになる拡大率
            int i;
            for (i = multitable.Length - 1; i > 0; i--)
            {
                if (multitable[i] < m) break;  // 横幅いっぱいより小さい拡大率を探す
            }
            sttime = (time0 < time1) ? time0 : time1;
            sttime -= (int)((gw - wavex - 18) / multitable[i] - diff) / 2;
            // ０以外から表示する場合は画面センターにする
            Multi(i);
        }


        // 表示位置左移動
        public void MoveLeft()
        {
            setStartTime((int)(-divpix / multi - 0.5));
        }


        // 表示位置右移動
        public void MoveRight()
        {
            setStartTime((int)(divpix / multi + 0.5));
        }


        // 全体⇔拡大 表示倍率切り替え
        public void MagFull()
        {
            if (imultifull == imulti)
            {  // 拡大率が同じなら全体表示中とする
                sttime = prevsttime;
                Multi(previmulti);
            }
            else
            {
                prevsttime = sttime;
                previmulti = imulti;
                ZoomBetween(0, wavelength);
                imultifull = imulti;
                sttime = 0;
            }
            UpdatePaint();
        }


        // カーソルをセンターにして画面表示する
        public void movetoCursor(TimeCursor cur)
        {
            sttime = cur.time - (int)((gw - wavex - 18) / multi / 2);
            UpdatePaint();
        }


        // すべてのカーソルをデータ範囲内にする
        public void SetCursorInArea()
        {
            foreach (TimeCursor cur in curlist)
            {
                if (cur.time < 0) cur.time = 0;
                if (cur.time >= wavelength) cur.time = wavelength - 1;
            }
        }


        // カーソルハイライト
        public void CursorHighlight(TimeCursor c)
        {
            c.Highlight();
            foreach (TimeCursor cur in curlist)
            {
                if (cur != c) cur.Normal();
            }
        }


        // 新しい波形を挿入
        public void WaveInsert(int waveno, TimeGraphWave newwave)
        {
            //mainf.debugprint( "WaveInsert\r" );
            WaveNormal();    // 今までのハイライトを戻す
            WaveRefresh();
            UpdatePaint();
            wavelist.Insert(waveno, newwave);
            WaveRefresh();
            UpdatePaint();
        }


        // 波形色選択
        public void WaveColor(int waveno)
        {
            ((TimeGraphWave)wavelist[waveno]).penWave.Color = clr.colordialog(((TimeGraphWave)wavelist[waveno]).penWave.Color);
            UpdatePaint();
        }


        public void WaveNormal()
        {
            if (Highlight >= 0)
            {
                ((TimeGraphWave)wavelist[Highlight]).Normal();    // 今までのハイライトを戻す
            }
            Highlight = -1;
        }


        public void WaveMove(int wavenofrom, int wavenoto)
        {
            WaveNormal();    // 今までのハイライトを戻す
            wavelist.Insert(wavenoto, wavelist[wavenofrom]);
            if (wavenofrom > wavenoto)
            {
                wavelist.RemoveAt(wavenofrom + 1);
            }
            else
            {
                wavelist.RemoveAt(wavenofrom);
            }
            WaveRefresh();
            UpdatePaint();
        }


        public void WaveDelete(int waveno)
        {
            if (waven == 1) return;  // 最後の1個は消去できない
            WaveNormal();             // 今までのハイライトを戻す
            wavelist.RemoveAt(waveno);
            WaveRefresh();
            UpdatePaint();
        }


        // クリックされた波形番号を返す
        public int ClickedWave(int mousex, int mousey)
        {
            if (mousex > gx + gw - 18) return -1;
            if (mousex < gx) return -1;
            for (int i = wavebegin; i < waven; i++)
            {
                if (((TimeGraphWave)wavelist[i]).isInside(mousex, mousey)) return i;
            }
            return -1;                          // 波形以外のときは -1 を返す
        }


        //  波形番号、名前を再設定する
        public void WaveRefresh()
        {
            int y1 = wavey;
            waven = wavelist.Count;
            foreach (TimeGraphWave w in wavelist)
            {
                y1 += w.h;
                w.setname();
            }
            wavey2 = y1;
            vScrollBar1.Maximum = wavey2;        // 波形表示の最後y位置
        }


        // ２つの波形の上下の位置を交換する
        private void WaveSwap(int waveno1, int waveno2)
        {
            if ((waveno1 >= 0) && (waveno1 < waven) && (waveno2 >= 0) && (waveno2 < waven))
            {
                ((TimeGraphWave)wavelist[waveno1]).Normal();
                TimeGraphWave oldwave = (TimeGraphWave)wavelist[waveno1];
                wavelist[waveno1] = wavelist[waveno2];
                wavelist[waveno2] = oldwave;
                ((TimeGraphWave)wavelist[waveno2]).Highlight();
                WaveRefresh();
                UpdatePaint();
            }
        }


        // 波形解析
        protected void WaveAnalysis(TimeGraphWave wave)
        {
            long mask = wave.mask; //((LogianaWave)view.wavelist[waveno]).mask;
            long old = wavedata[0] & mask;
            double hw = 0;                    // H パルス幅累積
            double lw = 0;                    // L パルス幅累積
            double pw = 0;                    // 測定中のパルス幅
            int h2l = 0;                     // H から L への遷移回数
            int l2h = 0;                     // L から H への遷移回数
            for (int i = 1; i < wavelength; i++)
            {    // 全波形データ
                pw++;
                long data = wavedata[i] & mask;
                if (data != old)
                {             // 波形変化点
                    if (data == 0)
                    {             // 立下り
                        if (l2h != 0) hw += pw;     // L パルスとして有効なら足しこむ
                        h2l++;
                    }
                    else
                    {                        // 立上り
                        if (h2l != 0) lw += pw;
                        l2h++;
                    }
                    pw = 0;
                }
                old = data;
            }
            if (old == 0)
            {                  // 最後の変化点は無効なので差し引く
                h2l--;
            }
            else
            {
                l2h--;
            }
            double hwidth = 0, lwidth = 0, t = 0, freq = 0, duty = 0;
            if (l2h > 0) hwidth = hw / l2h * clkperiod;
            if (h2l > 0) lwidth = lw / h2l * clkperiod;
            if ((h2l > 0) && (l2h > 0))
            {
                t = lwidth + hwidth;
                freq = 1 / t;
                duty = hwidth / t * 100;
            }
            string strfreq = "　周波数　　：　" + util.Time2FreqStr(t, 1) + "　\n";
            string strt = "", strh = "", strl = "", strduty = "", unit = "";
            if (clkperiod < 1e-6)
            {
                t *= 1e9;
                hwidth *= 1e9;
                lwidth *= 1e9;
                unit = "ns";
            }
            else if (clkperiod < 1e-3)
            {
                t *= 1e6;
                hwidth *= 1e6;
                lwidth *= 1e6;
                unit = "us";
            }
            else
            {
                t *= 1e3;
                hwidth *= 1e3;
                lwidth *= 1e3;
                unit = "ms";
            }
            strt = String.Format("　周期　　　　：　{0:# ##0.##} {1}　\n", t, unit);
            strh = String.Format("　Ｈパルス幅　：　{0:# ##0.##} {1}　\n", hwidth, unit);
            strl = String.Format("　Ｌパルス幅　：　{0:# ##0.##} {1}　\n", lwidth, unit);
            strduty = String.Format("　Ｄｕｔｙ　　　 ：　{0:####0} %　\n", duty, unit);
            MessageBox.Show(strfreq + strt + strh + strl + strduty, "波形解析 " + wave.name);
        }



        //-------------------------------------------------------------------------
        // タイムスケール表示クラス
        //-------------------------------------------------------------------------
        public class TimeScale
        {
            private TimeGraphForm graph;
            private Bitmap bitmap;
            private int h;                       // タイムスケール高さ
            private Brush timefontcolor = new SolidBrush(clr.TimeName);       // 時間フォント色
            private Brush timescalebgcolor = new SolidBrush(clr.TimeBG);    // 時間スケール背景色
            private Pen timescalecolor = new Pen(clr.TimeLine);  // 時間スケール線色
            private Pen trgmarkcolor = new Pen(clr.TrgMark);  // 時間スケールトリガーマーク色
            private Brush trgfontcolor = new SolidBrush(clr.TrgMark);  // 時間スケールトリガーマークフォント色
            static public LogianaScaleDiv[] scaletbl = new LogianaScaleDiv[27] {
                new LogianaScaleDiv(  10, "ns",   10e-9, 2,   "5ns" ),
                new LogianaScaleDiv(  20, "ns",   20e-9, 4,   "5ns" ),
                new LogianaScaleDiv(  50, "ns",   50e-9, 5,  "10ns" ),
                new LogianaScaleDiv( 100, "ns",  100e-9, 5,  "20ns" ),
                new LogianaScaleDiv( 200, "ns",  200e-9, 4,  "50ns" ),
                new LogianaScaleDiv( 500, "ns",  500e-9, 5, "100ns" ),
                new LogianaScaleDiv(   1, "us",    1e-6, 5, "0.2us" ),
                new LogianaScaleDiv(   2, "us",    2e-6, 4, "0.5us" ),
                new LogianaScaleDiv(   5, "us",    5e-6, 5,   "1us" ),
                new LogianaScaleDiv(  10, "us",   10e-6, 5,   "2us" ),
                new LogianaScaleDiv(  20, "us",   20e-6, 4,   "5us" ),
                new LogianaScaleDiv(  50, "us",   50e-6, 5,  "10us" ),
                new LogianaScaleDiv( 100, "us",  100e-6, 5,  "20us" ),
                new LogianaScaleDiv( 200, "us",  200e-6, 4,  "50us" ),
                new LogianaScaleDiv( 500, "us",  500e-6, 5, "100us" ),
                new LogianaScaleDiv(   1, "ms",    1e-3, 5, "0.2ms" ),
                new LogianaScaleDiv(   2, "ms",    2e-3, 4, "0.5ms" ),
                new LogianaScaleDiv(   5, "ms",    5e-3, 5,   "1ms" ),
                new LogianaScaleDiv(  10, "ms",   10e-3, 5,   "2ms" ),
                new LogianaScaleDiv(  20, "ms",   20e-3, 4,   "5ms" ),
                new LogianaScaleDiv(  50, "ms",   50e-3, 5,  "10ms" ),
                new LogianaScaleDiv( 100, "ms",  100e-3, 5,  "20ms" ),
                new LogianaScaleDiv( 200, "ms",  200e-3, 4,  "50ms" ),
                new LogianaScaleDiv( 500, "ms",  500e-3, 5, "100ms" ),
                new LogianaScaleDiv(   1,  "s",     1  , 5,  "0.2s" ),
                new LogianaScaleDiv(   2,  "s",     2  , 4,  "0.5s" ),
                new LogianaScaleDiv(   5,  "s",     5  , 5,    "1s" )
            };


            // タイムスケールコンストラクタ
            public TimeScale(TimeGraphForm graph, int h)
            {
                this.graph = graph;
                this.h = h;
                bitmap = new Bitmap(1, 1);        // 
            }


            // 時間軸表示bitmap作成
            public void Update()
            {
                bitmap.Dispose();                 // 今までのビットマップを捨てて
                bitmap = new Bitmap(graph.gw - 18, h);  // 新しくサイズ変更したビットマップをつくる
                Graphics gr = Graphics.FromImage(bitmap);
                gr.FillRectangle(timescalebgcolor, 0, 0, graph.gw - 18, h);
                int xs = graph.wavex - (int)(graph.sttime * graph.multi)
                         + (int)(((graph.sttime * graph.multi) / graph.divpix) + 0.9999) * graph.divpix;
                int time = (int)(graph.sttime * graph.multi / graph.divpix + 0.9999) * graph.divtime;
                while (xs < graph.gw - 18)
                {           // スケール縦線表示
                    gr.DrawLine(timescalecolor, xs, h - 5, xs, h);
                    gr.DrawString(String.Format("{0}{1}", time, graph.timeunit), fnt.TGTime, timefontcolor, xs - 8, h - 18);
                    xs += graph.divpix;
                    time += (int)graph.divtime;
                }
                int trgmark = (int)((graph.triggerpoint - graph.sttime) * graph.multi) + graph.wavex;
                if ((trgmark >= graph.wavex) && (trgmark <= graph.gw))
                {
                    gr.DrawLine(trgmarkcolor, trgmark, 15, trgmark, h);
                    gr.DrawString("T", fnt.TGTime, trgfontcolor, trgmark-5, 0);
                }
                gr.DrawString(graph.subdiv + "/div", fnt.TGTime, timefontcolor, 10, h - 18);
            }


            // 時間軸表示
            public void Disp(Graphics gr)
            {
                gr.DrawImage(bitmap, graph.gx, graph.gy);
            }


            // マウスがタイムスケール上にあるか
            public bool isOnTimeScale(int x1, int y1)
            {
                return ((x1 >= graph.gx) && (y1 <= graph.gy + h));
            }


            //-------------------------------------------------------------------------
            // タイムスケールの 1div と実時間の対応クラス
            //-------------------------------------------------------------------------
            public class LogianaScaleDiv
            {
                public int div;                 // 1 div あたりの表示時間
                public string unit;             // 1 div あたりの表示時間の単位
                public double time;             // 1 div あたりの実時間
                public int nsubdiv;             // 1 div をさらに何個に分割するか
                public string subdiv;           // xxs/div 表示


                // コンストラクタ
                public LogianaScaleDiv(int div, string unit, double time, int nsubdiv, string subdiv)
                {
                    this.div = div;
                    this.unit = unit;
                    this.time = time;
                    this.nsubdiv = nsubdiv;
                    this.subdiv = subdiv;
                }
            }


        } // TimeScale 



        //-------------------------------------------------------------------------
        // 時間カーソル表示クラス
        //-------------------------------------------------------------------------
        public class TimeCursor
        {
            public TimeGraphForm graph;
            public string name;               // カーソル名前
            public int pos = 0;                 // 名前表示位置　0: 下側　1: 上側
            public int time = 0;              // カーソル位置（サンプリング時間単位　0-4095)
            public int cursorx = 0;           // カーソル位置（ピクセル単位）
            public Pen cursorpen = new Pen(clr.Cursor, 2);  // カーソル色
            public SolidBrush cursornamebgbrush = new SolidBrush(clr.CursorNameBG);
            // カーソル名前背景色
            public SolidBrush cursornamebrush = new SolidBrush(clr.CursorName);
            // カーソル名前色


            // コンストラクタ
            public TimeCursor(TimeGraphForm graph, string name, int pos)
            {
                this.graph = graph;
                this.name = name;
                this.pos = pos;
                Normal();
            }


            // 標準色にする
            public void Normal()
            {
                cursorpen.Color = clr.Cursor;    // カーソル色
                cursorpen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;    // 
                cursornamebgbrush.Color = clr.CursorNameBG;  // カーソル名前背景色
                cursornamebrush.Color = clr.CursorName;      // カーソル名前色
            }


            // ハイライト色にする
            public void Highlight()
            {
                cursorpen.Color = clr.CursorHighlight;    // カーソル色
                cursorpen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;    // 
                cursornamebgbrush.Color = clr.CursorHighlightNameBG;  // カーソル名前背景色
                cursornamebrush.Color = clr.CursorHighlightName;      // カーソル名前色
            }


            // カーソル表示（サンプリング時間）
            public void Disp(Graphics gr, int time)
            {
                this.time = time;
                if (time >= graph.sttime)
                {
                    cursorx = (int)((time - graph.sttime) * graph.multi + 0.5) + graph.gx + graph.wavex;
                    if (cursorx < graph.gx + graph.gw - 18)
                    {
                        gr.DrawLine(cursorpen, cursorx, graph.gy + graph.wavey, cursorx, graph.wavey1);
                        string str = name + " : " + (time * graph.clkperiod * 1e9).ToString(); // name
                        int w = 7 * str.Length + 6;  // 13
                        int y = (pos == 0) ? graph.gy + 2 : graph.gy + graph.wavey - 15;
                        gr.FillRectangle(cursornamebgbrush, cursorx - w / 2, y, w, 13);
                        gr.DrawString(str, fnt.TGCursor, cursornamebrush, cursorx - w / 2, y);
                        //mainf.debugprint( String.Format( "\rDisp   cursorx-w/2:{0}  y:{1} ",cursorx-w/2, y )+str );
                    }
                }
            }


            // カーソルマークの上か
            public bool isOnCursor(int x1, int y1)
            {
                if (y1 <= graph.gy) return false;
                if ((pos == 0) && (y1 < graph.gy + 15)) return true;
                if ((pos == 1) && (y1 >= graph.gy + 15) && (y1 <= graph.gy + graph.wavey)) return true;
                if (y1 >= graph.gy + graph.wavey) return ((x1 >= cursorx - 8) && (x1 <= cursorx + 9));
                return false;
            }

        }

        private void TimeGraphForm_Click(object sender, EventArgs e)
        {
            this.Focus();
        }



    }
}
