﻿//================================================================================
//  
//        TimeGraphWavePropertyForm.cs       タイムスケールグラフ波形プロパティ
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど商業的利用はおやめください。
//


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace APB_3 {
    public partial class TimeGraphWavePropertyForm : Form {
        private TimeGraphWave wave;
        private Color clrWave;
        private Color clrWaveValue;

        public TimeGraphWavePropertyForm(TimeGraphWave wave) {
            this.wave = wave;
            InitializeComponent();
            comboBox_msb.Items.AddRange(def.datastrtbl);
            comboBox_width.Items.AddRange(def.widthstrtbl);
            comboBox_radix.Items.AddRange(def.radixstrtbl);
            comboBox_height.Items.AddRange(def.heightstrtbl);

            textBox_name.Text = wave.name;

            long mask = wave.mask;
            int bit = 0;
            for (bit = 63; bit > 0; bit--) {
                if (((ulong)mask & 0x8000000000000000L) != 0) break;
                mask = mask << 1;
            }
            int nbit = 0;
            while (mask != 0) {
                nbit++;
                mask = mask << 1;
            }
            comboBox_msb.SelectedIndex = bit;
            comboBox_width.Text = nbit.ToString();

            int radix = wave.radix;
            int i = 0;
            switch (radix) {
                case 2: i = 0; break;
                case 8: i = 1; break;
                case 10: i = 2; break;
                case 11: i = 3; break;  // 符号付き１０進数
                case 16: i = 4; break;
                case 17: i = 5; break;  // 符号なしアナログ表示
                case 18: i = 6; break;  // 符号付きアナログ表示
                default: i = 4; break;  // デフォールトは16進表示
            }
            comboBox_radix.SelectedIndex = i;

            comboBox_height.Text = wave.h.ToString();

            clrWave = wave.penWave.Color;
            clrWaveValue = wave.brWaveValue.Color;
        }

        private void button_Cancel_Click(object sender, EventArgs e) {
            Close();
        }

        private void button_OK_Click(object sender, EventArgs e) {
            if (textBox_name.Text != wave.name) {
                wave.name = textBox_name.Text;
                wave.nameedited = true;
            }

            int bit = comboBox_msb.SelectedIndex;
            int nbit = util.ToInt32(comboBox_width.Text);
            long mask = 0;
            long m = 1L << bit;
            for (int i = 0; i < nbit; i++) {
                mask |= m;
                m = m >> 1;
            }
            wave.mask = mask;

            // 基数
            int r = comboBox_radix.SelectedIndex;
            int radix = 2;
            switch (r) {
                case 0: radix = 2; break;
                case 1: radix = 8; break;
                case 2: radix = 10; break;
                case 3: radix = 11; break;  // 符号付き１０進数
                case 4: radix = 16; break;
                case 5: radix = 17; break;  // 符号なしアナログ表示
                case 6: radix = 18; break;  // 符号付きアナログ表示
            }
            wave.radix = radix;

            // 高さ
            int h = util.ToInt32(comboBox_height.Text);
            wave.h = (h > 5) ? h : 5;

            wave.penWave.Color = clrWave;
            wave.brWaveValue.Color = clrWaveValue;
            Close();
        }

        private void button_Color_Click(object sender, EventArgs e) {
            clrWave = clr.colordialog(clrWave);
        }

        private void button_Vcolor_Click(object sender, EventArgs e) {
            clrWaveValue = clr.colordialog(clrWaveValue);
        }

        private void button_Color_Paint(object sender, PaintEventArgs e) {
            Graphics gr = e.Graphics;
            gr.FillRectangle(new SolidBrush(clrWave), 3, 3, 15, 15);
        }

        private void button_Vcolor_Paint(object sender, PaintEventArgs e) {
            Graphics gr = e.Graphics;
            gr.FillRectangle(new SolidBrush(clrWaveValue), 3, 3, 15, 15);
        }

    }
}
