----------------------------------------------------------------------------------
--  
--        cordic_sine.vhd        CORDIC gĈʑTCvZilatency=19j
--  
--        (c)2009/07/26  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷ȂǏƓIp͂߂B
--
-- Number of Slices:                      136  out of   2448     5%  
-- Number of Slice Flip Flops:             71  out of   4896     1%  
-- Number of 4 input LUTs:                256  out of   4896     5%  
--
--   Minimum period: 8.512ns (Maximum Frequency: 117.484MHz)
--   Minimum input arrival time before clock: 4.666ns
--   Maximum output required time after clock: 4.040ns
--   Maximum combinational path delay: No path found
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

------------------------------------------------------------------------------
entity cordic_sine is 
  port (
    CLK     : in  std_logic;                       -- 100MHz
    ND      : in  std_logic;                       -- ʑ̓Cl[u
    PHASE   : in  std_logic_vector( 15 downto 0 ); -- ʑ
    SINE_OUT: out std_logic_vector( 15 downto 0 ); -- TCo
    OD      : out std_logic                        -- TCo̓Cl[u
  );
end cordic_sine;


------------------------------------------------------------------------------
architecture RTL of cordic_sine is
  subtype D16 is STD_LOGIC_VECTOR (15 downto 0);
  type ATNTABLE is array (0 to 15) of D16;
  constant ATN :ATNTABLE := (    -- Atan(2^-i)
       X"4000", X"25C8", X"13F6", X"0A22", X"0516", X"028C", X"0146", X"00A3",
       X"0051", X"0029", X"0014", X"000A", X"0005", X"0003", X"0001", X"0001"
  );
  
  
  signal X0 : std_logic_vector( 15 downto 0 ); -- 
  signal Y0 : std_logic_vector( 15 downto 0 ); -- 
  signal X  : std_logic_vector( 15 downto 0 ); -- 
  signal Y  : std_logic_vector( 15 downto 0 ); -- 
  signal Z  : std_logic_vector( 15 downto 0 ); -- 
  signal N  : std_logic_vector(  4 downto 0 ); -- 
  signal XN : std_logic_vector( 15 downto 0 ); -- N bit Vtg
  signal YN : std_logic_vector( 15 downto 0 ); -- N bit Vtg
  signal QUAD : std_logic_vector( 1 downto 0 ); -- ی
  
  component shiftra16
  port (
    X   : in  std_logic_vector( 15 downto 0 ); -- 
    Y   : out std_logic_vector( 15 downto 0 ); -- o̓f[^
    N   : in  std_logic_vector( 3 downto 0 )   -- Vtg
  );
  end component;
  
  
  begin
    
    u0 : shiftra16
    port map (
      X   => X,             -- 
      Y   => XN,            -- o̓f[^
      N   => N(3 downto 0)  -- Vtg
    );
    
    
    u1 : shiftra16
    port map (
      X   => Y,             -- 
      Y   => YN,            -- o̓f[^
      N   => N(3 downto 0)  -- Vtg
    );
    
    
    QUAD <= PHASE(15 downto 14);
    
    X0 <= X"0000" when(QUAD="01" or QUAD="11") else
          X"4DB0" when(QUAD="00") else                --  19888=32752/1.64676 I[o[t[Ȃ悤 16 Ă
          X"B250" ;                                   -- -19888
    
    Y0 <= X"0000" when(QUAD="00" or QUAD="10") else
          X"4DB0" when(QUAD="01") else                --  19888
          X"B250" ;                                   -- -19888
    
    
    ------------------------------------------------------------------
    --    CORDIC ŃTCvZ
    ------------------------------------------------------------------
    process( CLK ) begin
      if( rising_edge(CLK) ) then
        if( ND='1' ) then
          X <= X0;
          Y <= Y0;
          Z <= "0" & PHASE(13 downto 0) & "0";
          N <= (others =>'0');
        elsif( N(4)='0' ) then
          if( Z(15)='0' ) then          -- Z >= 0
            X <= X - YN;
            Y <= Y + XN;
            Z <= Z - ATN(CONV_INTEGER(N(3 downto 0)));
          else                          -- Z < 0
            X <= X + YN;
            Y <= Y - XN;
            Z <= Z + ATN(CONV_INTEGER(N(3 downto 0)));
          end if;
          N <= N+1;
        else
          if( N(0)='0' ) then
            SINE_OUT <= Y;                -- SINE
            OD <= '1';
            N <= N+1;
          else
            OD <= '0';
          end if;
        end if;
      end if;
    end process;
    
    
  end RTL;
----------------------------- end of architecture ----------------------------

