----------------------------------------------------------------------------------
--  
--        dds0.vhd        ccriTCARTĈQo  phase:32bits  out:18bitsj
--  
--        (c)2009/07/26  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷ȂǏƓIp͂߂B
--
--
--Selected Device : 3s200avq100-4 
--
-- Number of Slices:                       57  out of   1792     3%  
-- Number of Slice Flip Flops:            106  out of   3584     2%  
-- Number of 4 input LUTs:                110  out of   3584     3%  
-- Number of IOs:                          69
-- Number of bonded IOBs:                  69  out of     68   101% (*) 
-- Number of BRAMs:                         2  out of     16    12%  
-- Number of MULT18X18SIOs:                 4  out of     16    25%  
-- Number of GCLKs:                         1  out of     24     4%  
--
--   Minimum period: 5.538ns (Maximum Frequency: 180.571MHz)
--   Minimum input arrival time before clock: 5.738ns
--   Maximum output required time after clock: 5.531ns
--   Maximum combinational path delay: No path found
--
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

Library UNISIM;
use UNISIM.vcomponents.all;

------------------------------------------------------------------------------
entity dds0 is
  port (
    DDSCLK      : in  std_logic;
    FREQ        : in  std_logic_vector(31 downto 0);    -- g
    RESET_PHASE : in  std_logic;                        -- ʑZbg
    COS         : out std_logic_vector(17 downto 0);    -- RTCo
    SIN         : out std_logic_vector(17 downto 0)     -- TCo
  );
end dds0;

------------------------------------------------------------------------------
architecture RTL of dds0 is
  
  signal PHASE  : STD_LOGIC_VECTOR (31 downto 0);
  signal PHASEH : STD_LOGIC_VECTOR ( 8 downto 0);
  signal PHASEL : STD_LOGIC_VECTOR ( 8 downto 0);
  signal ROM_MAIN  : STD_LOGIC_VECTOR (35 downto 0);
  signal ROM_SUB   : STD_LOGIC_VECTOR (35 downto 0);
  
  signal ms1_out, ms2_out : STD_LOGIC_VECTOR (35 downto 0);
  signal SIN1 : STD_LOGIC_VECTOR (21 downto 0);
  signal mc1_out, mc2_out : STD_LOGIC_VECTOR (35 downto 0);
  signal COS1 : STD_LOGIC_VECTOR (21 downto 0);
  
  
  component rom_sin_upper
  port (
    clka  : IN  std_logic;
    addra : IN  std_logic_VECTOR(8 downto 0);
    douta : OUT std_logic_VECTOR(35 downto 0)
  );
  end component;
  
  component rom_sin_lower
  port (
    clka  : IN  std_logic;
    addra : IN  std_logic_VECTOR(8 downto 0);
    douta : OUT std_logic_VECTOR(35 downto 0)
  );
  end component;
  
  
  
  begin
    
    rom0 : rom_sin_upper
    port map (
      addra => PHASEH,
      clka  => DDSCLK,
      douta => ROM_MAIN
    );
    
    rom1 : rom_sin_lower
    port map (
      addra => PHASEL,
      clka  => DDSCLK,
      douta => ROM_SUB
    );
    
    ms1 : MULT18X18S
    port map (
      P => ms1_out, -- 36-bit multiplier output
      A => ROM_MAIN(17 downto 0), -- 18-bit multiplier input
      B => ROM_SUB(35 downto 18), -- 18-bit multiplier input
      C => DDSCLK, -- Clock input
      CE => '1', -- Clock enable input  
      R => '0' -- Synchronous reset input
    );  
        
    ms2 : MULT18X18S
    port map (
      P => ms2_out, -- 36-bit multiplier output
      A => ROM_MAIN(35 downto 18), -- 18-bit multiplier input
      B => ROM_SUB(17 downto 0), -- 18-bit multiplier input
      C => DDSCLK, -- Clock input
      CE => '1', -- Clock enable input  
      R => '0' -- Synchronous reset input
    );  
    
    mc1 : MULT18X18S
    port map (
      P => mc1_out, -- 36-bit multiplier output
      A => ROM_MAIN(17 downto 0), -- 18-bit multiplier input
      B => ROM_SUB(17 downto 0), -- 18-bit multiplier input
      C => DDSCLK, -- Clock input
      CE => '1', -- Clock enable input  
      R => '0' -- Synchronous reset input
    );  
        
    mc2 : MULT18X18S
    port map (
      P => mc2_out, -- 36-bit multiplier output
      A => ROM_MAIN(35 downto 18), -- 18-bit multiplier input
      B => ROM_SUB(35 downto 18), -- 18-bit multiplier input
      C => DDSCLK, -- Clock input
      CE => '1', -- Clock enable input  
      R => '0' -- Synchronous reset input
    );  
    
    
    PHASEH <= PHASE(31 downto 23);
    PHASEL <= PHASE(22 downto 14);
    
    
    
    ------------------------------------------------------------------
    --    ʑAL[^[
    ------------------------------------------------------------------
    process( RESET_PHASE, DDSCLK ) begin
      if( rising_edge(DDSCLK) ) then
        if( RESET_PHASE = '1' ) then
          PHASE <= (others => '0');
        else
          PHASE <= PHASE + FREQ;
        end if;
      end if;
    end process;
    
    
    ------------------------------------------------------------------
    --    rhm
    ------------------------------------------------------------------
    process( DDSCLK ) begin
      if( rising_edge(DDSCLK) ) then
        SIN1 <= ms1_out(34 downto 13) + ms2_out(34 downto 13);
        SIN  <= SIN1(21 downto 4) + ("00000000000000000"&SIN1(3));
      end if;
    end process;
    
    
    ------------------------------------------------------------------
    --    bnr
    ------------------------------------------------------------------
    process( DDSCLK ) begin
      if( rising_edge(DDSCLK) ) then
        COS1 <= mc1_out(34 downto 13) - mc2_out(34 downto 13);
        COS  <= COS1(21 downto 4) + ("00000000000000000"&COS1(3));
      end if;
    end process;
    
    
    
  end RTL;
----------------------------- end of architecture ----------------------------
