----------------------------------------------------------------------------------
--  
--        dds1.vhd        ccriTCo  phase:32bits  out:18bitsj
--  
--        (c)2009/07/26  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷ȂǏƓIp͂߂B
--multi18x18s latency=2 ɂ
-- Number of Slices:                      230  out of   2448     9%  
-- Number of Slice Flip Flops:            179  out of   4896     3%  
-- Number of 4 input LUTs:                425  out of   4896     8%  
--
--   Minimum period: 9.745ns (Maximum Frequency: 102.614MHz)
--   Minimum input arrival time before clock: 10.366ns
--   Maximum output required time after clock: 4.040ns
--   Maximum combinational path delay: No path found
--
--
--multi18x18s latency=5
-- Number of Slices:                      238  out of   2448     9%  
-- Number of Slice Flip Flops:            467  out of   4896     9%  
-- Number of 4 input LUTs:                455  out of   4896     9%  
--    Number used as logic:               425
--    Number used as Shift registers:      30
--
--   Minimum period: 4.734ns (Maximum Frequency: 211.247MHz)
--   Minimum input arrival time before clock: 5.355ns
--   Maximum output required time after clock: 4.040ns
--   Maximum combinational path delay: No path found
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

Library UNISIM;
use UNISIM.vcomponents.all;

------------------------------------------------------------------------------
entity dds1 is
  port (
    DDSCLK  : in  std_logic;
    FREQ    : in  std_logic_vector(31 downto 0);    -- g
    SIG     : out std_logic_vector(17 downto 0)     -- Mo
  );
end dds1;

------------------------------------------------------------------------------
architecture RTL of dds1 is
  
  signal PHASE  : STD_LOGIC_VECTOR (31 downto 0);
  signal PHASEH : STD_LOGIC_VECTOR ( 8 downto 0);
  signal PHASEL : STD_LOGIC_VECTOR ( 8 downto 0);
  signal ROM_MAIN  : STD_LOGIC_VECTOR (35 downto 0);
  signal ROM_SUB   : STD_LOGIC_VECTOR (35 downto 0);
  
  signal ms1_out, ms2_out : STD_LOGIC_VECTOR (35 downto 0);
  signal SIN1 : STD_LOGIC_VECTOR (21 downto 0);
  
  signal SIN2 : STD_LOGIC_VECTOR (17 downto 0);
  signal mul_out : STD_LOGIC_VECTOR (35 downto 0);
  
  
  component rom_sin_upper
  port (
    clka  : IN  std_logic;
    addra : IN  std_logic_VECTOR(8 downto 0);
    douta : OUT std_logic_VECTOR(35 downto 0)
  );
  end component;
  
  component rom_sin_lower
  port (
    clka  : IN  std_logic;
    addra : IN  std_logic_VECTOR(8 downto 0);
    douta : OUT std_logic_VECTOR(35 downto 0)
  );
  end component;
  
  component multi18x18s
  	port (
  	clk: IN std_logic;
  	a: IN std_logic_VECTOR(17 downto 0);
  	b: IN std_logic_VECTOR(17 downto 0);
  	p: OUT std_logic_VECTOR(35 downto 0));
  end component;
  
  begin
    
    rom0 : rom_sin_upper
    port map (
      addra => PHASEH,
      clka  => DDSCLK,
      douta => ROM_MAIN
    );
    
    rom1 : rom_sin_lower
    port map (
      addra => PHASEL,
      clka  => DDSCLK,
      douta => ROM_SUB
    );
    
    ms1 : MULT18X18S
    port map (
      P => ms1_out, -- 36-bit multiplier output
      A => ROM_MAIN(17 downto 0), -- 18-bit multiplier input
      B => ROM_SUB(35 downto 18), -- 18-bit multiplier input
      C => DDSCLK, -- Clock input
      CE => '1', -- Clock enable input  
      R => '0' -- Synchronous reset input
    );  
        
    ms2 : MULT18X18S
    port map (
      P => ms2_out, -- 36-bit multiplier output
      A => ROM_MAIN(35 downto 18), -- 18-bit multiplier input
      B => ROM_SUB(17 downto 0), -- 18-bit multiplier input
      C => DDSCLK, -- Clock input
      CE => '1', -- Clock enable input  
      R => '0' -- Synchronous reset input
    );  
    
    SIG <= SIN2;  -- ŏIo͔g`
    
    PHASEH <= PHASE(31 downto 23);
    PHASEL <= PHASE(22 downto 14);
    
    
    ------------------------------------------------------------------
    --    ʑAL[^[
    ------------------------------------------------------------------
    process( DDSCLK ) begin
      if( rising_edge(DDSCLK) ) then
        if( FREQ(31)='0' ) then         -- normal mode
          PHASE <= PHASE + FREQ;
        else                            -- test mode
          PHASE <= PHASE + "00100000000000000000000000000000"; -- =12.5MHz(100MHz/8)
        end if;
      end if;
    end process;
    
    
    ------------------------------------------------------------------
    --    rhm
    ------------------------------------------------------------------
    process( DDSCLK ) begin
      if( rising_edge(DDSCLK) ) then
        if( FREQ(31)='0' ) then         -- normal mode
          SIN1 <= ms1_out(34 downto 13) + ms2_out(34 downto 13);
          SIN2 <= SIN1(21 downto 4) + ("00000000000000000"&SIN1(3));
        else                            -- test mode
          if( PHASE(31)='1' ) then
            SIN2 <= FREQ(17 downto 0);
          else
            SIN2 <= (others => '0');
          end if;
        end if;
      end if;
    end process;
    
    
  end RTL;
----------------------------- end of architecture ----------------------------
