------------------------------------------------------------------------------
--
--       ft232h_asyncfifo.vhd          Interface with FT232_H (245 ASYNC FIFO mode)
--
------------------------------------------------------------------------------
--
--  READ ( ADR  Auto increment  j
--    0    : 0x5C         read R}h ( Module  FT232_H  PC )
--    1..2 : address      adr_l + adr_h
--    3..4 : size         size_l + size_h 0:65536oCgA1-65535:1`65535oCg
--    ̌ size oCg̃f[^W[ǂݏo PC ɓ]
--    
--  READ ( ADR ͌Œ j
--    0    : 0x5D         read R}h ( Module  FT232_H  PC )
--    1..2 : address      adr_l + adr_h
--    3..4 : size         size_l + size_h 0:65536oCgA1-65535:1`65535oCg
--    ̌ size oCg̃f[^W[ǂݏo PC ɓ]
--    
--    
--  WRITE ( ADR  Auto increment  j
--    0    : 0x54         write R}h ( PC  FT232_H  Module )
--    1..2 : address      adr_l + adr_h
--    3..4 : size         size_l + size_h 0:65536oCgA1-65535:1`65535oCg
--    5..n : data0..n     size oCg̃f[^AW[ɏ
--    
--  WRITE ( ADR ͌Œ j
--    0    : 0x55         write R}h ( PC  FT232_H  Module )
--    1..2 : address      adr_l + adr_h
--    3..4 : size         size_l + size_h 0:65536oCgA1-65535:1`65535oCg
--    5..n : data0..n     size oCg̃f[^AW[ɏ
--    
--    
--  DF_RW ( DF ɏ݁ADF ǂ񂾃f[^ DF_BUF ɏ j
--    0    : 0x44         
--    1..2 : size         size_l + size_h 1-512 oCg
--    f[^ DF(SI) ɑƓ DF(SO) ǂݎ DF_BUF ɏ
--    
--  DF_BUF_READ ( DF_BUF ̃f[^ǂݏo j DFRW s DF ǂ񂾃f[^ DF_BUF ɓ̂łǂ
--    0    : 0x45         
--    1..2 : size         size_l + size_h 1-512 oCg
--    ̌ DF_BUF ̃AhX 0 f[^ǂݏo PC ɓ]
--    
----  DF_BUF_WRITE ( DF_BUF Ƀf[^ j Ƀf[^Ă DFRW s
----    0    : 0x46         
----    1..2 : size         size_l + size_h 1-512 oCg
----    3..n : data0..n     size oCg̃f[^A DF_BUF ̃AhX 0 珑
----    
------------------------------------------------------------------------------
--
--Selected Device : 6slx9tqg144-3 
--
--
--Slice Logic Utilization: 
-- Number of Slice Registers:              61  out of  11440     0%  
-- Number of Slice LUTs:                  143  out of   5720     2%  
--    Number used as Logic:               143  out of   5720     2%  
--
--Slice Logic Distribution: 
-- Number of LUT Flip Flop pairs used:    150
--   Number with an unused Flip Flop:      89  out of    150    59%  
--   Number with an unused LUT:             7  out of    150     4%  
--   Number of fully used LUT-FF pairs:    54  out of    150    36%  
--   Number of unique control sets:        12
--
--   Minimum period: 3.604ns (Maximum Frequency: 277.458MHz)
--   Minimum input arrival time before clock: 6.153ns
--   Maximum output required time after clock: 3.701ns
--   Maximum combinational path delay: 4.566ns
--


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

------------------------------------------------------------------------------
entity ft232h is
  port (
    RESET       : in  std_logic;
    CLK         : in  std_logic;                      -- I/O bus clock
    
    FT232_RXF_X : in  std_logic;                      -- FT232 RXF#
    FT232_TXE_X : inout std_logic;                    -- FT232 TXE#   (out DF_SCK)
    FT232_RD_X  : out std_logic;                      -- FT232 RD_X#
    FT232_WR_X  : out std_logic;                      -- FT232 WR_X#
    FT232_SIWU_X: out std_logic;                      -- FT232 SIWU#  send immediate
    FT232_OE_X  : out std_logic;                      -- FT232 OE#        Async FIFO ł͖gp
    FT232_CLK   : in  std_logic;                      -- FT232 CLKOUT     Async FIFO ł͖gp
    FT232_DATA  : inout std_logic_vector(7 downto 0); -- FT232 DATA   (D7:DF_SO)
    
    DF_CS_X     : out std_logic;                      -- AT25DF041 CS#          P38
    DF_SI       : out std_logic;                      -- AT25DF041 SI           P64
    
    MWE         : out std_logic;                      -- Module Write Enable
    MRE         : out std_logic;                      -- Module Read Enable
    MCE         : out std_logic_vector(15 downto 0 ); -- Module Enable
    MWDATA      : out std_logic_vector( 7 downto 0 ); -- W[݃f[^
    MRDATA      : in  std_logic_vector( 7 downto 0 ); -- W[ǂݏof[^
    MADR        : out std_logic_vector(15 downto 0 ); -- W[AhX
    
    FTDEBUG    : out std_logic_vector( 4 downto 0 )  -- DEBUG
    
  );
end ft232h;

------------------------------------------------------------------------------
architecture RTL of ft232h is
  
  signal RD_X       : std_logic;
  signal WR_X       : std_logic;
  signal RXF_X      : std_logic;
  signal TXE_X      : std_logic;
  signal TXE1_X     : std_logic;
  signal RD         : std_logic;
  signal RDEN       : std_logic;
  signal RE         : std_logic;
  signal RW         : std_logic;
  signal D_DIR      : std_logic;
  signal SIWU_X     : std_logic;
  signal AUTOINC    : std_logic;
  signal ADR        : std_logic_vector(15 downto 0);
  signal SIZE       : std_logic_vector(15 downto 0);
  signal SIZERW     : std_logic_vector( 8 downto 0);
  signal DF         : std_logic_vector( 1 downto 0); -- DataFlash [h
  signal DF_CS1_X   : std_logic;                     -- DF_CS_X
  signal DF_Z       : std_logic;                     -- DF [h HiZ ɂ|[gRg[
  signal DF_WDATA   : std_logic_vector( 7 downto 0); -- DF ɏf[^
  signal DF_RDATA   : std_logic_vector( 7 downto 0); -- DF ǂ񂾃f[^ADF_BUF ɏ
  signal DF_BITCNT  : std_logic_vector( 4 downto 0); -- bit counter
  
  signal WEA        : std_logic_vector( 0 downto 0); -- DF_BUF [݃^C~O
  signal WEB        : std_logic_vector( 0 downto 0); -- DF_BUF [݃^C~O
  signal DOUTA      : std_logic_vector( 7 downto 0); -- DF_BUF [ǂݏof[^
  signal DOUTB      : std_logic_vector( 7 downto 0); -- DF_BUF [ǂݏof[^
  
  
  signal FTSTATE : integer range 0 to 15;
  constant FT_IDLE : integer := 0;
  constant FT_RD1  : integer := 1;
  constant FT_RD2  : integer := 2;
  constant FT_RD3  : integer := 3;
  constant FT_RD4  : integer := 4;
  constant FT_RD5  : integer := 5;
  constant FT_RD6  : integer := 6;
  constant FT_WR1  : integer := 7;
  constant FT_WR2  : integer := 8;
  constant FT_WR3  : integer := 9;
  constant FT_WR4  : integer := 10;
  constant FT_WR5  : integer := 11;
  constant FT_WR6  : integer := 12;
  constant FT_SI1  : integer := 13;
  constant FT_SI2  : integer := 14;
  constant FT_SI3  : integer := 15;
  
  signal CMDSTATE : integer range 0 to 31;
  constant CMD_IDLE   : integer := 0;
  constant CMD_ADR1   : integer := 1;
  constant CMD_ADR2   : integer := 2;
  constant CMD_SIZE1  : integer := 3;
  constant CMD_SIZE2  : integer := 4;
  constant CMD_RD1    : integer := 5;
  constant CMD_RD2    : integer := 6;
  constant CMD_RD3    : integer := 7;
  constant CMD_WR1    : integer := 8;
  constant CMD_WR2    : integer := 9;
  
  constant CMD_DF1    : integer := 15;
  constant CMD_DFBWR1 : integer := 16;
  constant CMD_DFBWR2 : integer := 17;
  constant CMD_DFBWR3 : integer := 18;
  constant CMD_DFRW0  : integer := 19;
  constant CMD_DFRW1  : integer := 20;
  constant CMD_DFRW2  : integer := 21;
  constant CMD_DFRW3  : integer := 22;
  constant CMD_DFBRD1 : integer := 23;
  constant CMD_DFBRD2 : integer := 24;
  constant CMD_DFBRD3 : integer := 25;
  
  
  COMPONENT ram512_8
    PORT (
      clka  : IN STD_LOGIC;
      wea   : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
      addra : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
      dina  : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      douta : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
      clkb  : IN STD_LOGIC;
      web   : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
      addrb : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
      dinb  : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      doutb : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
    );
  END COMPONENT;
  
  
begin
  
  ram : ram512_8              -- DF R/W BUFFER
    PORT MAP (
      clka  => CLK,
      wea   => WEA,
      addra => ADR( 8 downto 0 ),
      dina  => DF_RDATA,
      douta => DOUTA,
      clkb  => CLK,
      web   => WEB,
      addrb => ADR( 8 downto 0 ),
      dinb  => FT232_DATA,
      doutb => DOUTB
    );
  
  MCE    <=  "0000000000000000" when( DF_CS1_X='0'            ) else  -- DF ANZX
             "0000000000000000" when( ADR(15)='1'             ) else  -- memory R/W       0x8000-0xFFFF
             "0000000000000000" when( ADR(14 downto 12) /= "000" ) else  -- 0xxx ȊO
             "0000000000000001" when( ADR(11 downto 8) = X"0" ) else  -- signature        00xx
             "0000000000000010" when( ADR(11 downto 8) = X"1" ) else  -- 
             "0000000000000100" when( ADR(11 downto 8) = X"2" ) else  -- 
             "0000000000001000" when( ADR(11 downto 8) = X"3" ) else  -- adc1610s         03xx
             "0000000000010000" when( ADR(11 downto 8) = X"4" ) else  -- dds              04xx
             "0000000000100000" when( ADR(11 downto 8) = X"5" ) else  -- 
             "0000000001000000" when( ADR(11 downto 8) = X"6" ) else  -- 
             "0000000010000000" when( ADR(11 downto 8) = X"7" ) else  -- memorycontrol    07xx
             "0000000100000000" when( ADR(11 downto 8) = X"8" ) else  -- freqcounter      08xx
             "0000001000000000" when( ADR(11 downto 8) = X"9" ) else  -- 
             "0000010000000000" when( ADR(11 downto 8) = X"A" ) else  -- demod
             "0000100000000000" when( ADR(11 downto 8) = X"B" ) else  -- 
             "0001000000000000" when( ADR(11 downto 8) = X"C" ) else  -- 
             "0010000000000000" when( ADR(11 downto 8) = X"D" ) else  -- 
             "0100000000000000" when( ADR(11 downto 8) = X"E" ) else  -- 
             "1000000000000000";                                      -- 
  
  FT232_RD_X  <= RD_X;-- when( DF_Z='0' ) else 'Z';
  FT232_WR_X  <= WR_X;
  FT232_DATA  <= (others => 'Z') when( D_DIR = '0'    ) else
                 DOUTB           when( CMDSTATE = CMD_DFBRD3 ) else  -- DF_BUF_READ
                 MRDATA;                                       -- MRDATA
  
  FT232_SIWU_X <= SIWU_X;
  
  FT232_OE_X  <= '0';         -- Asynchronous FIFO ł͕svASynchronous FIFO ł͕Kv
  
  DF_SI       <= DF_WDATA(7);                               -- SI = MSB first
  DF_CS_X     <= DF_CS1_X;
  FT232_TXE_X <= DF_BITCNT(1) when( DF_CS1_X='0' ) else 'Z';-- DF SCLK o 25MHz
  
  MWDATA      <= FT232_DATA;
  MADR        <= ADR;
  MRE         <= RE;
  
  
  
  
  ------------------------------------------------------------------
  --    DEBUG
  ------------------------------------------------------------------
  FTDEBUG(0) <= WR_X;
  FTDEBUG(1) <= SIWU_X;
  FTDEBUG(2) <= '0' when( CMDSTATE = CMD_IDLE ) else '1';
  FTDEBUG(3) <= '0' when( SIZE  = "000000000" ) else '1';
  FTDEBUG(4) <= '0' when( FTSTATE = FT_IDLE ) else '1';
  
  
  ------------------------------------------------------------------
  --    FT232_ M̓
  ------------------------------------------------------------------
  process( RESET, CLK ) begin
--    if ( RESET ='1') then
--      RXF_X <= '1';
--      TXE_X <= '1';
--    elsif( rising_edge(CLK) ) then
    if( rising_edge(CLK) ) then
      if ( RESET ='1') then
        RXF_X <= '1';
        TXE_X <= '1';
      else
        RXF_X <= FT232_RXF_X;
        TXE_X <= FT232_TXE_X;
        TXE1_X <= TXE_X;
      end if;
    end if;
  end process;
  
  
  -- WRITE ( FT232H f[^ǂ݁AW[ɏ@@PC ݂ Write ) -----------------------------
  --
  --      CLK _|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__ 10ns(100MHz)
  --    RXF_X ~~|_________________________________________|~~~~~~~~~~ 
  --     RD_X ~~~~~~~~|_______________________|~~~~~~~~~~~~~~~~~~~~~~ width=min30ns
  --     D_IN -----------14ns------------------------ =MWDATA
  --      RD _____________________|~~~~~|____________________________ 
  --  FTSTATE   IDLE  | WR1 | WR2 | WR3 | WR4 | WR5 | WR6 | IDLE 
  --
  --     MWE ___________________________|~~~~~|______________________
  --     ADR                                  | ADR+1
  -- CMDSTATE                       WR1 | WR2 | WR1(IDLE) 
  --------------------------------------------------------------------------------------
  
  -- READ ( W[f[^ǂ݁AFT232H ɏ@@PC ݂ Read ) -----------------------------
  --
  --      CLK _|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__|~~|__
  --    TXE_X ~~|_______________________|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
  --    D_OUT -------------------------------------
  --      DIR ______________|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|______________________
  --     WR_X ~~~~~~~~~~~~~~~~~~~~|_______________________|~~~~~~~~~~~~~~~~~~~~~~ width=min30ns
  --   SIWU_X ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|________________|~~~~~ Ō̓]̂
  --  FTSTATE   IDLE        | RD1 | RD2 | RD3 | RD4 | RD5 | SI1 | SI2 | IDLE 
  --                                                        IDLE      
  --
  --      RE  ________|~~~~~|___________________________________|~~~~~|_____
  --      ADR                                                   | ADR+1
  -- CMDSTATE    RD1  | RD2 | RD3(IDLE)                         | RD2
  --------------------------------------------------------------------------------------
  
  ------------------------------------------------------------------
  --    FT232H READ/WRITE 
  ------------------------------------------------------------------
  process( RESET, CLK ) begin
    if ( RESET = '1') then
      RD_X     <= '1';
      RD       <= '0';
      WR_X     <= '1';
      D_DIR    <= '0';
      SIWU_X   <= '1';
      FTSTATE  <= FT_IDLE;
    elsif( rising_edge(CLK) ) then
        case FTSTATE is
          when FT_IDLE => ------------------------------------------------
            if( RXF_X = '0' and RDEN = '1' ) then -- start write process
              RD_X    <= '0';
              FTSTATE <= FT_WR1;
            elsif( TXE_X = '0' and RE = '1' ) then -- start read process
              D_DIR   <= '1';
              FTSTATE <= FT_RD1;
            end if;
            SIWU_X <= '1';
            
          when FT_WR1 => ------------------------------------------------
            FTSTATE <= FT_WR2;
          when FT_WR2 => ------------------------------------------------
            RD      <= '1';
            FTSTATE <= FT_WR3;
          when FT_WR3 => ------------------------------------------------
            RD      <= '0';
            FTSTATE <= FT_WR4;
          when FT_WR4 => ------------------------------------------------
            RD_X    <= '1';
            FTSTATE <= FT_WR5;
          when FT_WR5 => ------------------------------------------------
            FTSTATE <= FT_WR6; -- RXF# ̒x{̒x l
          when FT_WR6 => ------------------------------------------------
            FTSTATE <= FT_IDLE;
          
          when FT_RD1 => ------------------------------------------------
            WR_X    <= '0';
            FTSTATE <= FT_RD2;
          when FT_RD2 => ------------------------------------------------
            FTSTATE <= FT_RD3;
          when FT_RD3 => ------------------------------------------------
            FTSTATE <= FT_RD4;
          when FT_RD4 => ------------------------------------------------
            FTSTATE <= FT_RD5;
          when FT_RD5 => ------------------------------------------------
            WR_X    <= '1';
            D_DIR   <= '0';
            if( CMDSTATE = CMD_IDLE ) then
              if( TXE_X = '0' ) then
                SIWU_X <= '0';
                FTSTATE   <= FT_SI1;
              end if;
            else
              FTSTATE   <= FT_IDLE;
            end if;
          
          when FT_SI1 => ------------------------------------------------
            FTSTATE   <= FT_SI2;
          when FT_SI2 => ------------------------------------------------
            FTSTATE   <= FT_IDLE;
          
          when others => ------------------------------------------------------
            FTSTATE <= FT_IDLE; -- 肦ȂXe[g
        end case;
    end if;
  end process;
  
  
  --------------------------------------------------------------------
  --     R}hߎsi@W[ǂݏ@DF ǂݏ@j
  --------------------------------------------------------------------
  process( RESET, CLK ) begin
    if( RESET = '1' ) then
      CMDSTATE <= CMD_IDLE;
      RE   <= '0';
      MWE  <= '0';
      RDEN <= '1';
      DF   <= (others => '0');
      DF_CS1_X  <= '1';
      DF_BITCNT <= (others => '0');
    elsif( rising_edge(CLK) ) then
        case CMDSTATE is
          when CMD_IDLE => -----------------------------------------------
            if( RD = '1' ) then
              if( FT232_DATA(7 downto 1)&"0" = X"5C" ) then    -- READ( Module  FT232H  PC )
                DF <= (others => '0');
                RW <= '1';  -- Read
                CMDSTATE   <= CMD_ADR1;
              elsif( FT232_DATA(7 downto 1)&"0" = X"54" ) then -- WRITE( PC  FT232H  Module )
                DF <= (others => '0');
                RW <= '0';  -- Write
                CMDSTATE   <= CMD_ADR1;
              elsif( FT232_DATA(7 downto 0) = X"44" ) then    --  DFRW ( DF ɏ݁ADF ǂ񂾃f[^ DF_BUF ɏj
                DF <= "10";  -- DF
                CMDSTATE   <= CMD_SIZE1;
              elsif( FT232_DATA(7 downto 0) = X"45" ) then    --  DF_BUF_READ ( DF_BUF ̃f[^ǂݏo j
                DF <= "11";  -- DF
                CMDSTATE   <= CMD_SIZE1;
              end if;
              AUTOINC <= FT232_DATA(0);   -- '0' ȂAhXCNg
            end if;                       -- R}hłȂƂ͖
            
            RDEN <= '1';                  -- M
            ADR <= (others => '0');       
            DF_BITCNT <= (others => '0'); 
          
          when CMD_ADR1 => -----------------------------------------------
            if( RD = '1' ) then
              ADR(7 downto 0) <= FT232_DATA;
              CMDSTATE   <= CMD_ADR2;
            end if;
          when CMD_ADR2 => -----------------------------------------------
            if( RD = '1' ) then
              ADR(15 downto 8) <= FT232_DATA;
              CMDSTATE   <= CMD_SIZE1;
            end if;
          
          when CMD_SIZE1 => -----------------------------------------------
            if( RD = '1' ) then
              SIZE(7 downto 0) <= FT232_DATA;
              CMDSTATE   <= CMD_SIZE2;
            end if;
          when CMD_SIZE2 => -----------------------------------------------
            if( RD = '1' ) then
              SIZE(15 downto 8) <= FT232_DATA;
              if( DF(1) = '1' ) then      -- DataFlash [h
                CMDSTATE   <= CMD_DF1;
              elsif( RW = '1' ) then      -- RW ŃXe[g𕪂
                RDEN <= '0';              -- M RXF_X  '0' ɂȂĂMȂB
                CMDSTATE   <= CMD_RD1;
              else
                CMDSTATE   <= CMD_WR1;
              end if;
            end if;
          
          
          -- ========== READ =============================================
          when CMD_RD1 => ------------------------------------------------
            if( TXE_X = '0' and FTSTATE = FT_IDLE ) then
              RE <= '1';                         -- W[ǂݏopX
              CMDSTATE   <= CMD_RD2;
            end if;
          when CMD_RD2 => ------------------------------------------------
            RE <= '0';                         -- W[ǂݏopX
            SIZE <= SIZE - 1;
            if( SIZE = X"0001" ) then
              RDEN <= '1';
              CMDSTATE <= CMD_IDLE;        -- ]I
            else
              CMDSTATE <= CMD_RD3;         -- ̃f[^]҂
            end if;
          when CMD_RD3 => ------------------------------------------------
            if( TXE_X = '0' and FTSTATE = FT_IDLE ) then  -- ADR  +1 ^C~O҂
              if( AUTOINC = '0' ) then
                ADR <= ADR + 1;
              end if;
              RE <= '1';                   -- W[ǂݏopX
              CMDSTATE <= CMD_RD2;         -- ̃f[^]Jn
            end if;
          
          
          -- ========== WRITE ============================================
          when CMD_WR1 => ------------------------------------------------
            if( RD = '1' ) then
              MWE <= '1';                      -- W[݃pX
              CMDSTATE   <= CMD_WR2;
            end if;
          when CMD_WR2 => ------------------------------------------------
            if( AUTOINC = '0' ) then
              ADR <= ADR + 1;
            end if;
            MWE <= '0';
            SIZE <= SIZE - 1;
            if( SIZE = X"0001" ) then
              CMDSTATE <= CMD_IDLE;       -- ]I
            else
              CMDSTATE <= CMD_WR1;        -- ̃f[^]
            end if;
          
          
          -- ========== DF ==============================================
          when CMD_DF1 => ------------------------------------------------
            if( FTSTATE = FT_IDLE ) then            -- FT232H ̃f[^]I҂
              if( DF(0) = '0' ) then    -- DF_RW
                SIZERW <= SIZE( 8 downto 0);        -- DF ǂݏsize
                CMDSTATE   <= CMD_DFBWR1;
              else                                  -- DF_BUF_WRITE
                RDEN <= '0';                        -- ǂݍݒ RXF_X  '0' ɂȂĂMȂB
                CMDSTATE   <= CMD_DFBRD1;
              end if;
            end if;
          
          
          -- ========== DF_RW ==============================================
          -- ܂ DF_BUF  FT232 ̃f[^
          when CMD_DFBWR1 => ------------------------------------------------
            if( RD = '1' ) then
              WEB <= "1";               -- DF_BUF ݃pX
              SIZE <= SIZE - 1;
              CMDSTATE   <= CMD_DFBWR2;
            end if;
          when CMD_DFBWR2 => ------------------------------------------------
            ADR <= ADR + 1;
            WEB <= "0";
            if( SIZE = X"0000" ) then
              CMDSTATE <= CMD_DFRW0;    -- f[^ DF_BUF ɏÎ DF ANZXJn
            else
              CMDSTATE <= CMD_DFBWR1;   -- ̃f[^]
            end if;
            
          --  DF_BUF ̓e DF SI ɓ]Ɠ SO ǂ񂾃f[^ DF_BUF ɏ
          when CMD_DFRW0 => ------------------------------------------------
            if( FTSTATE = FT_IDLE ) then  -- FT232H ̃f[^]I҂
              DF_CS1_X <= '0';            -- DF ANZXJn
              ADR <= (others => '0');
              RDEN <= '0';                -- DF ANZX RXF_X  '0' ɂȂĂMȂB
              CMDSTATE <= CMD_DFRW1;
            end if;
          when CMD_DFRW1 => ------------------------------------------------
            SIZERW <= SIZERW -1;
            CMDSTATE <= CMD_DFRW2;
          when CMD_DFRW2 => ------------------------------------------------
            DF_WDATA <= DOUTA;
            CMDSTATE <= CMD_DFRW3;
          when CMD_DFRW3 => ------------------------------------------------
            if( DF_BITCNT(1 downto 0) = "11" ) then
              DF_WDATA <= DF_WDATA(6 downto 0) & "0";           -- DF_SI
            elsif( DF_BITCNT(1 downto 0) = "10" ) then          -- |[góADF o͂̒xl 1 NbNǂݎ^C~Ox点Ă
              DF_RDATA <= DF_RDATA(6 downto 0) & FT232_DATA(7); -- =DF_SO
            end if;
            
            if( DF_BITCNT = "11110" ) then
              WEA <= "1";
            elsif( DF_BITCNT = "11111" ) then
              WEA <= "0";
              ADR <= ADR + 1;
              if( SIZERW = X"0000" ) then
                DF_CS1_X <= '1';          -- DF ANZXI
                CMDSTATE <= CMD_IDLE;     
              else
                CMDSTATE <= CMD_DFRW1;
              end if;
            end if;
            
            DF_BITCNT <= DF_BITCNT + 1;
            
          -- ========== DF_BUF_READ =========================================
          when CMD_DFBRD1 => ------------------------------------------------
            if( TXE_X = '0' and FTSTATE = FT_IDLE ) then
              RE <= '1';                  -- FT232H ǂݏoJn
              SIZE <= SIZE - 1;
              CMDSTATE <= CMD_DFBRD2;
            end if;
          when CMD_DFBRD2 => ------------------------------------------------
            RE <= '0';                    
              CMDSTATE <= CMD_DFBRD3;     -- ̃f[^]҂
          when CMD_DFBRD3 => ------------------------------------------------
            if( TXE_X = '0' and FTSTATE = FT_IDLE ) then  -- ADR  +1 ^C~O҂
              if( SIZE = X"0000" ) then
                CMDSTATE <= CMD_IDLE;       -- ]I
              else
                ADR <= ADR + 1;
                CMDSTATE <= CMD_DFBRD1;     -- ̃f[^]҂
              end if;
            end if;
          
          
          when others => ------------------------------------------------------
            CMDSTATE   <= CMD_IDLE;       -- 肦ȂXe[g
        end case;
    end if;
  end process;
  
  
end RTL;
----------------------------- end of architecture ----------------------------
