// file: test_exdes.v
// (c) Copyright 2009 - 2011 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.

//----------------------------------------------------------------------------
// SelectIO wizard example design
//----------------------------------------------------------------------------
// This example design instantiates the IO circuitry
//----------------------------------------------------------------------------

`timescale 1ps/1ps

module test_exdes
   // Clock -> q modeling delay
 #(parameter TCQ = 100,
   // width of the data for the system
   parameter sys_w = 8,
   // width of the data for the device
   parameter dev_w = 16,
   // width of the address for the memory
   parameter add_w = 10,
   // depth of the memory
   parameter add_l = 1 << add_w)
  (
   // Memory interface
   input      [add_w-1:0] ADDRESS_USER,
   input      [dev_w-1:0] DATA_IN_USER,
   output reg [dev_w-1:0] DATA_OUT_USER,
   input                  ENABLE_USER,
   input                  WRITE_USER,
   input                  ENABLE_DEVICE,
   // From the system into the device
   input      [sys_w-1:0] DATA_IN_FROM_PINS_P,
   input      [sys_w-1:0] DATA_IN_FROM_PINS_N,
   input                  CLK_IN_P,
   input                  CLK_IN_N,

   input                  CLK_RESET,
   input                  IO_RESET);


   // connection between ram and io circuit
   wire [dev_w-1:0]       DATA_IN_TO_DEVICE;
   wire                   write_device    = 1'b1;
   reg [add_w-1:0]        address_device  = 0;
   reg [dev_w-1:0]        ram_array [0:add_l-1];
   wire                   CLK_OUT;
   wire                   DELAY_BUSY;
   wire                   DELAY_CLK       = CLK_OUT;
   reg                    DELAY_DATA_CAL  = 0;
   reg                    DELAY_DATA_CE   = 0;
   reg                    DELAY_DATA_INC  = 0;
   reg                    DELAY_TAP_IN    = 5'b00000;
   reg                    DELAY_RESET     = 0;
//   reg                    DELAY_CLK       = 0;



   // Infer a dual-port memory
   always @(posedge CLK_OUT) begin
     if (ENABLE_USER) begin
       if (WRITE_USER) begin
         ram_array[ADDRESS_USER] <= #TCQ DATA_IN_USER;
       end
       DATA_OUT_USER <= #TCQ ram_array[ADDRESS_USER];
     end
   end

   always @(posedge CLK_OUT) begin
     if (ENABLE_DEVICE) begin
       if (write_device) begin
         ram_array[address_device] <= #TCQ DATA_IN_TO_DEVICE;
       end
     end
   end

   // auto-increment access into the RAM on the IO side
   always @(posedge CLK_OUT) begin
     if (IO_RESET) begin
       address_device <= #TCQ 0;
     end else if (ENABLE_DEVICE) begin
       address_device <= #TCQ address_device + 1'b1;
     end
   end

   // Instantiate the IO design
   test io_inst
    (
     // From the system into the device
     .DATA_IN_FROM_PINS_P     (DATA_IN_FROM_PINS_P),
     .DATA_IN_FROM_PINS_N     (DATA_IN_FROM_PINS_N),
     .DATA_IN_TO_DEVICE       (DATA_IN_TO_DEVICE),
     .DELAY_BUSY              (DELAY_BUSY),
     .DELAY_CLK               (DELAY_CLK),
     .DELAY_DATA_CAL          (1'b0),
     .DELAY_DATA_CE           (1'b0),
     .DELAY_DATA_INC          (1'b0),
     .CLK_IN_P                (CLK_IN_P),
     .CLK_IN_N                (CLK_IN_N),
     .CLK_OUT                 (CLK_OUT),
     .CLK_RESET               (CLK_RESET),
     .IO_RESET                (IO_RESET));

endmodule



