-- file: test_exdes.vhd
-- (c) Copyright 2009 - 2011 Xilinx, Inc. All rights reserved.
-- 
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
-- 
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
-- 
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
-- 
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.

------------------------------------------------------------------------------
-- SelectIO wizard example design
------------------------------------------------------------------------------
-- This example design instantiates the IO circuitry
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library unisim;
use unisim.vcomponents.all;

entity test_exdes is
generic (
  -- Clock -> q modeling delay
  TCQ        : time    := 100 ps;
  -- width of the data for the system
  sys_w      : integer := 8;
  -- width of the data for the device
  dev_w      : integer := 16;
  -- width of the address for the memory
  add_w      : integer := 10;
  -- depth of the memory
  add_l      : integer := 1024);
port (
  -- Memory interface
  ADDRESS_USER             : in    std_logic_vector(add_w-1 downto 0);
  DATA_IN_USER             : in    std_logic_vector(dev_w-1 downto 0);
  DATA_OUT_USER            : out   std_logic_vector(dev_w-1 downto 0);
  ENABLE_USER              : in    std_logic;
  WRITE_USER               : in    std_logic;
  ENABLE_DEVICE            : in    std_logic;
  -- From the system into the device
  DATA_IN_FROM_PINS_P      : in    std_logic_vector(sys_w-1 downto 0);
  DATA_IN_FROM_PINS_N      : in    std_logic_vector(sys_w-1 downto 0);
  CLK_IN_P                 : in    std_logic;
  CLK_IN_N                 : in    std_logic;
  CLK_RESET                : in    std_logic;
  IO_RESET                 : in    std_logic);
end test_exdes;

architecture xilinx of test_exdes is

component test is
generic
 (-- width of the data for the system
  sys_w       : integer := 8;
  -- width of the data for the device
  dev_w       : integer := 16);
port
 (
  -- From the system into the device
  DATA_IN_FROM_PINS_P     : in    std_logic_vector(sys_w-1 downto 0);
  DATA_IN_FROM_PINS_N     : in    std_logic_vector(sys_w-1 downto 0);
  DATA_IN_TO_DEVICE       : out   std_logic_vector(dev_w-1 downto 0);

-- Input, Output delay control signals
  DELAY_BUSY              : out   std_logic;
  DELAY_CLK               : in    std_logic;
  DELAY_DATA_CAL          : in    std_logic;
  DELAY_DATA_CE           : in    std_logic;                    -- Enable signal for delay 
  DELAY_DATA_INC          : in    std_logic;                    -- Delay increment (high), decrement (low) signal
-- Clock and reset signals
  CLK_IN_P                : in    std_logic;                    -- Differential fast clock from IOB
  CLK_IN_N                : in    std_logic;
  CLK_OUT                 : out   std_logic;
  CLK_RESET               : in    std_logic;                    -- Reset signal for Clock circuit
  IO_RESET                : in    std_logic);                   -- Reset signal for IO circuit
end component;

   -- This example design doesn't use the variable delay programming
   signal DELAY_BUSY           : std_logic;
   signal DELAY_CLK            : std_logic := '0';
   signal DELAY_DATA_CAL       : std_logic := '0';
   signal DELAY_DATA_CE        : std_logic := '0';
   signal DELAY_RESET          : std_logic := '0';
   signal DELAY_DATA_INC       : std_logic := '0';
   signal DELAY_TAP_IN         : std_logic_vector (4 downto 0) := "00000";
            -- connection between ram and io circuit
   signal DATA_IN_TO_DEVICE    : std_logic_vector(dev_w-1 downto 0);
   signal write_device         : std_logic :=  '1';
   signal address_device       : std_logic_vector(add_w-1 downto 0) := (others=>'0');
   type ram_type is array (0 to add_l-1) of std_logic_vector(dev_w-1 downto 0);
   shared variable ram_array   : ram_type;
   signal CLK_OUT              : std_logic;
begin
   DELAY_CLK <= CLK_OUT;


   -- Infer a dual port memory
   process (CLK_OUT) begin
   if (CLK_OUT = '1' and CLK_OUT'event) then
     if (ENABLE_USER = '1') then
       if (WRITE_USER = '1') then
         ram_array(conv_integer(ADDRESS_USER)) := DATA_IN_USER;
       end if;
       DATA_OUT_USER <= ram_array(conv_integer(ADDRESS_USER))  after TCQ;
     end if;
   end if;
   end process;

   process (CLK_OUT) begin
   if (CLK_OUT = '1' and CLK_OUT'event) then
     if (ENABLE_DEVICE = '1') then
       if (write_device = '1') then
         ram_array(conv_integer(address_device)) := DATA_IN_TO_DEVICE;
       end if;
     end if;
   end if;
   end process;

   -- auto-increment access into the RAM on the IO side
   process (CLK_OUT) begin
   if (CLK_OUT = '1' and CLK_OUT'event) then
     if (IO_RESET = '1') then
       address_device <=  (others=>'0') after TCQ;
     elsif (ENABLE_DEVICE = '1') then
       address_device <= address_device + 1 after TCQ;
     end if;
   end if;
   end process;

   -- Instantiate the IO design
   io_inst : test
   port map
   (
    -- From the system into the device
    DATA_IN_FROM_PINS_P     => DATA_IN_FROM_PINS_P,
    DATA_IN_FROM_PINS_N     => DATA_IN_FROM_PINS_N,
    DATA_IN_TO_DEVICE       => DATA_IN_TO_DEVICE,

    DELAY_BUSY              => DELAY_BUSY,
    DELAY_CLK               => DELAY_CLK,
    DELAY_DATA_CAL            => '0',
    DELAY_DATA_CE             => '0',
    DELAY_DATA_INC            => '0',

    CLK_IN_P                => CLK_IN_P,
    CLK_IN_N                => CLK_IN_N,
    CLK_OUT                 => CLK_OUT,
    CLK_RESET               => CLK_RESET,
    IO_RESET                => IO_RESET);
end xilinx;



