// file: test_tb.v
// (c) Copyright 2009 - 2011 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.

//----------------------------------------------------------------------------
// SelectIO wizard demonstration testbench
//----------------------------------------------------------------------------
// This demonstration testbench instantiates the example design for the 
//   SelectIO wizard. 
//----------------------------------------------------------------------------

`timescale 1ps/1ps

module test_tb ();
  parameter        TCQ   = 100;
  parameter        sys_w = 8;
  parameter        dev_w = 16;
  parameter        add_w = 10;
  localparam       data_rate = 2;

  reg  [add_w-1:0] ADDRESS_USER;
  reg  [dev_w-1:0] DATA_IN_USER;
  wire [dev_w-1:0] DATA_OUT_USER;
  reg              ENABLE_USER;
  reg              WRITE_USER;
  reg              ENABLE_DEVICE;
  // From the system into the device
  reg  [sys_w-1:0] DATA_IN_FROM_PINS;
  wire [sys_w-1:0] DATA_IN_FROM_PINS_P =  DATA_IN_FROM_PINS;
  wire [sys_w-1:0] DATA_IN_FROM_PINS_N = ~DATA_IN_FROM_PINS;
  wire [sys_w-1:0] DATA_IN_FROM_PINS_DEL;
  wire [sys_w-1:0] DATA_IN_FROM_PINS_P_DEL;
  wire [sys_w-1:0] DATA_IN_FROM_PINS_N_DEL;
  reg              CLK_IN   = 0;
  wire             CLK_IN_P =  CLK_IN;
  wire             CLK_IN_N = ~CLK_IN;
  reg              CLK_RESET;
  reg              IO_RESET;

  // clock generator- 100 MHz simulation clock
  //------------------------------------------
  localparam       clk_per = 10 * 1000;
  always begin
    CLK_IN = #(clk_per/2) ~CLK_IN;
  end

  // Test sequence
  reg [15*8-1:0] test_phase = "";
  integer        slice_count;
  integer        ii;

  initial begin
    test_phase = "reset";
    DATA_IN_FROM_PINS = 0;
    // reset the logic
    CLK_RESET   = 1;
    IO_RESET    = 1;
    CLK_RESET   = 0;
    #(18*clk_per);
    WRITE_USER    = 0;
    ENABLE_USER   = 0;
    // start up the io
    IO_RESET      = 0;
    #(8*clk_per);
    ENABLE_DEVICE = 1;
    test_phase = "data_in";
    fork
    // Drive data onto the pins
    begin
      for (ADDRESS_USER = 0; ADDRESS_USER < 256; ADDRESS_USER = ADDRESS_USER + 1) begin
         DATA_IN_FROM_PINS = ADDRESS_USER;
         #(clk_per/data_rate);
      end
    end
    // Snoop data from the device
    begin
      #(clk_per*2/data_rate);
      for (ii = 0; ii < 256; ii = ii + 1) begin
        $display("Received %0h", dut.DATA_IN_TO_DEVICE);
        #(clk_per/data_rate);
      end
    end
    join
    // Set up any display statements using time to be readable
    $timeformat(-9, 2, "ns", 10);
    $display("SIMULATION PASSED");
    $display("SYSTEM_CLOCK_COUNTER : %0d\n",$time/clk_per);
    $finish;
  end
   assign #2500 DATA_IN_FROM_PINS_P_DEL = DATA_IN_FROM_PINS_P;
   assign #2500 DATA_IN_FROM_PINS_N_DEL = DATA_IN_FROM_PINS_N;

  // Instantiation of the example design
  //---------------------------------------------------------
  test_exdes
   #(.TCQ   (100),
     .sys_w (8),
     .dev_w (16),
     .add_w (10))
  dut
  (// Memory interface
   .ADDRESS_USER              (ADDRESS_USER),
   .DATA_IN_USER              (DATA_IN_USER),
   .DATA_OUT_USER             (DATA_OUT_USER),
   .ENABLE_USER               (ENABLE_USER),
   .WRITE_USER                (WRITE_USER),
   .ENABLE_DEVICE             (ENABLE_DEVICE),
   // From the system into the device
   .DATA_IN_FROM_PINS_P       (DATA_IN_FROM_PINS_P_DEL),
   .DATA_IN_FROM_PINS_N       (DATA_IN_FROM_PINS_N_DEL),
   .CLK_IN_P                  (CLK_IN_P),
   .CLK_IN_N                  (CLK_IN_N),
   .CLK_RESET                 (CLK_RESET),
   .IO_RESET                  (IO_RESET));
endmodule



