----------------------------------------------------------------------------------
--  
--        uda1754.vhd        Audio DAC (UDA1345) interface
--  
--        (c)2009/07/26  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷ȂǏƓIp͂߂B
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

------------------------------------------------------------------------------
entity uda1345 is 
  port (
    RESET   : in  std_logic;
    CLK     : in  std_logic;                      -- 100MHz
    MCE     : in  std_logic;                      -- Module Chip Enable
    MWE     : in  std_logic;                      -- Module Write Enable
    MRE     : in  std_logic;                      -- Module Read Enable
    MADR    : in  std_logic_vector(15 downto 0);  -- W[AhX
    MWDATA  : in  std_logic_vector(7 downto 0);   -- data bus input
    MRDATA  : out std_logic_vector(7 downto 0);   -- data bus output
    
    CNT32   : in  std_logic_vector(31 downto 0);  -- 32bit counter
    
    D0_IN   : in  std_logic_vector(15 downto 0);  -- ̓f[^ 0 ( DDS2 )
    D1_IN   : in  std_logic_vector(15 downto 0);  -- ̓f[^ 1 ( DDS3 )
    D2_IN   : in  std_logic_vector(15 downto 0);  -- ̓f[^ 2 ( DEMOD_OUT )
    D4_IN   : in  std_logic_vector(15 downto 0);  -- ̓f[^ 4 ( FMDEMOD_OUT )
    I_IN    : in  std_logic_vector(15 downto 0);  -- ̓f[^ 3 ( I M )
    Q_IN    : in  std_logic_vector(15 downto 0);  -- ̓f[^ 3 ( Q M )
    
    L_OUT   : out std_logic_vector(15 downto 0);  -- I[fBI Lch A/D o
    R_OUT   : out std_logic_vector(15 downto 0);  -- I[fBI Rch A/D o
    ND      : out std_logic;                      -- I[fBI A/D ϊo L
    
    SCK     : out std_logic;                      -- 256fs =25MHz     =CLK/4
    BCK     : out std_logic;                      -- 32fs  =3.125MHz  =CLK/32
    LRCK    : out std_logic;                      -- fs    =97.656KHz =CLK/1024  L='0' R='1'
    I2SOUT  : out std_logic;                      -- I2S data out (MSB first)
    I2SIN   : in  std_logic;                      -- I2S data in (MSB first)
    
    L3MODE  : out std_logic;                      -- MP2
    L3CLK   : out std_logic;                      -- MP3  2MHzmax
    L3DATA  : out std_logic                       -- MP4
    
  );
end uda1345;


------------------------------------------------------------------------------
architecture RTL of uda1345 is
  
  signal I2SOUT_L   : std_logic_vector(15 downto 0);  -- 
  signal I2SOUT_R   : std_logic_vector(15 downto 0);  -- 
  signal I2SOUT_LR  : std_logic_vector(31 downto 0);  -- 
  
  signal I2SIN_L    : std_logic_vector(15 downto 0);  -- 
  signal I2SIN_R    : std_logic_vector(15 downto 0);  -- 
  signal I2SIN_LR   : std_logic_vector(31 downto 0);  -- 
  
  signal I2SEN      : std_logic; -- 
  
  signal SEL_L      : std_logic_vector(2 downto 0);   -- 
  signal SEL_R      : std_logic_vector(2 downto 0);   -- 
  
  signal L3CLKEN    : std_logic; -- 
  signal STATUS_DATA: std_logic_vector(7 downto 0);   -- f[^
  signal L3DATAADRS : std_logic_vector(16 downto 0);  -- f[^ƃAhX@ɃVtĝłPrbg]
  signal L3BITCNT   : std_logic_vector(3 downto 0);   -- 
  signal START_STATUS: std_logic; -- 
  signal START_DATA : std_logic; -- 
  
  signal L3STATE    : integer range 0 to 3;
  constant L3_IDLE  : integer := 0;
  constant L3_ADR1  : integer := 1;
  constant L3_ADR2  : integer := 2;
  constant L3_DATA  : integer := 3;
  
  
begin
  
  I2SOUT_L <= (others => '0') when( SEL_L="000" ) else
              D0_IN           when( SEL_L="001" ) else
              D1_IN           when( SEL_L="010" ) else
              D2_IN           when( SEL_L="011" ) else
              Q_IN            when( SEL_L="100" ) else
              D4_IN           when( SEL_L="101" ) else
              I2SIN_L         when( SEL_L="110" ) else  -- for debug
              "1111111100000000";                       -- for debug
  
  I2SOUT_R <= (others => '0') when( SEL_R="000" ) else
              D0_IN           when( SEL_R="001" ) else
              D1_IN           when( SEL_R="010" ) else
              D2_IN           when( SEL_R="011" ) else
              I_IN            when( SEL_R="100" ) else
              D4_IN           when( SEL_R="101" ) else
              I2SIN_R         when( SEL_R="110" ) else  -- for debug
              "1000000000000000";                       -- for debug
  
  I2SEN <= '0' when( SEL_L="000" and SEL_R="000" ) else '1';  -- I2S Mo͂邩
  
  SCK   <= '0' when( I2SEN='0' ) else CNT32(1);   -- 256fs =25MHz     =CLK/4
  BCK   <= '0' when( I2SEN='0' ) else CNT32(4);   -- 32fs  =3.125MHz  =CLK/32
  LRCK  <= '0' when( I2SEN='0' ) else CNT32(9);   -- fs    =97.656KHz =CLK/1024
  I2SOUT<= '0' when( I2SEN='0' ) else I2SOUT_LR(31);-- I2S DATA =MSB first 
  
  ------------------------------------------------------------------
  --  I2S o
  ------------------------------------------------------------------
  process( CLK ) begin
    if (rising_edge (CLK)) then
      if( CNT32(4 downto 0) = "11111" ) then      -- BCLK ̗ŁA
        if( CNT32(9 downto 5) = "00000" ) then    -- LRCLK ɓăf[^Zbg
          I2SOUT_LR <= I2SOUT_L & I2SOUT_R;       -- f[^
        else
          I2SOUT_LR <= I2SOUT_LR(30 downto 0) & "X";  -- f[^Vtg
        end if;
      end if;
    end if;
  end process;
  
  
  ------------------------------------------------------------------
  --  I2S 
  ------------------------------------------------------------------
  L_OUT <= I2SIN_L;
  R_OUT <= I2SIN_R;
  
  process( CLK ) begin
    if (rising_edge (CLK)) then
      if( CNT32(4 downto 0) = "10000" ) then      -- BCLK ̗オŁA
        if( CNT32(9 downto 5) = "00000" ) then    -- LRCLK ɓăf[^b`
          I2SIN_L <= I2SIN_LR(30 downto 15) ;     -- 
          I2SIN_R <= I2SIN_LR(14 downto 0) & I2SIN; -- 
          ND  <= '1';                             -- f[^L
        else
          I2SIN_LR <= I2SIN_LR(30 downto 0) & I2SIN;-- f[^̓Vtg
        end if;
      else
        ND <= '0';
      end if;
    end if;
  end process;
  
  
  ------------------------------------------------------------------
  --  ݒ
  ------------------------------------------------------------------
  process( RESET, CLK ) begin
    if ( RESET ='1') then
      SEL_L <= (others => '0');   -- OFF
      SEL_R <= (others => '0');   -- OFF
    elsif( rising_edge(CLK) ) then
      if( MWE='1' and MCE='1' ) then
        if( MADR(1 downto 0)="00" ) then
          SEL_L <= MWDATA(6 downto 4);  -- MI@@k
          SEL_R <= MWDATA(2 downto 0);  -- MI@@q
        elsif( MADR(1 downto 0)="01" ) then
          STATUS_DATA  <= MWDATA;       -- L3 o̓f[^(STATUS)b`
          START_STATUS <= '1';
        else
          STATUS_DATA <= MWDATA;        -- L3 o̓f[^(DATA)b`
          START_DATA  <= '1';
        end if;
      else
        START_STATUS <= '0';
        START_DATA   <= '0';
      end if;
    end if;
  end process;
  
  
  ------------------------------------------------------------------
  --  L3 [ho
  ------------------------------------------------------------------
  L3DATA <= L3DATAADRS(0);                          -- LSB first
  L3CLK  <= CNT32(5) when( L3CLKEN='1' ) else '1';  -- 1.5625MHz
  
  process( RESET, CLK ) begin
    if ( RESET ='1') then
      L3MODE  <= '1';
      L3CLKEN <= '0';
      L3STATE <= L3_IDLE;
    elsif( rising_edge(CLK) ) then
      if( (L3STATE = L3_IDLE) and ((START_STATUS = '1') or ( START_DATA = '1' )) )then
        if( START_STATUS = '1') then
          L3DATAADRS <= STATUS_DATA & "000101100";
        else
          L3DATAADRS <= STATUS_DATA & "000101000";
        end if;
        L3STATE <= L3_ADR1;
      end if;
      
      if( CNT32(5 downto 0)="111110" ) then  -- L3CLK ̗
        case L3STATE is
          when L3_ADR1 =>
            L3MODE   <= '0';
            L3BITCNT <= (others => '0');
            L3STATE  <= L3_ADR2;
          when L3_ADR2 =>
            L3BITCNT   <= L3BITCNT + 1;
            if( L3BITCNT = "1000" ) then
              L3MODE   <= '1';
              L3CLKEN  <= '0';
              L3BITCNT <= (others => '0');
              L3STATE  <= L3_DATA;
            else
              L3CLKEN    <= '1';
              L3DATAADRS <= "X" & L3DATAADRS(16 downto 1);
            end if;
          when L3_DATA =>
            L3BITCNT   <= L3BITCNT + 1;
            if( L3BITCNT = "1000" ) then
              L3CLKEN  <= '0';
              L3STATE  <= L3_IDLE;
            else
              L3CLKEN    <= '1';
              L3DATAADRS <= "X" & L3DATAADRS(16 downto 1);
            end if;
          when others =>
        end case;
      end if;
    end if;
  end process;
  
  
end RTL;



----------------------------- end of architecture ----------------------------

